// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/driver/device.c $
// $Rev: 53598 $
// $Date: 2023-08-23 16:05:50 -0500 (Wed, 23 Aug 2023) $

// 18AI32SSC1M: Device Driver: source file

#include "main.h"



// variables ******************************************************************

const gsc_dev_id_t	dev_id_list[]	=
{
	// model			Vendor	Device	SubVen	SubDev	type
	{ "18AI32SSC1M",	0x10B5, 0x9056, 0x10B5, 0x3101,	GSC_DEV_TYPE_18AI32SSC1M	},
	{ "18AI32SSC1M",	0x10B5, 0x9056, 0x10B5, 0x3431,	GSC_DEV_TYPE_18AI32SSC1M	},
	{ NULL }
};



//*****************************************************************************
static void _channels_compute(dev_data_t* dev)
{
	switch (dev->cache.gsc_bcfgr_32 & 0x30000)
	{
		case 0x00000:	dev->cache.channel_qty	= 32;	break;
		case 0x10000:	dev->cache.channel_qty	= 16;	break;
		case 0x20000:	dev->cache.channel_qty	=  8;	break;
		case 0x30000:	dev->cache.channel_qty	=  4;	break;
		default:		dev->cache.channel_qty	= -1;	break;
	}

	dev->cache.channels_max	= 32;
}



//*****************************************************************************
static void _data_width_compute(dev_data_t* dev)
{
	if (dev->cache.gsc_bcfgr_32 & 0x100000)
		dev->cache.data_width	= 18;
	else
		dev->cache.data_width	= 16;
}



//*****************************************************************************
static void _ext_clk_div_compute(dev_data_t* dev)
{
	if ((dev->cache.gsc_bcfgr_32 & 0xFFF) >= 0x10)
		dev->cache.ext_clk_div	= 1;
	else
		dev->cache.ext_clk_div	= 0;
}



//*****************************************************************************
static void _master_clock_compute(dev_data_t* dev)
{
	if (dev->cache.gsc_bcfgr_32 & 0x40000)
		dev->cache.master_clock	= -1;
	else
		dev->cache.master_clock	= 36000000L;
}



//*****************************************************************************
static void _pretrigger_compute(dev_data_t* dev)
{
	if ((dev->cache.gsc_bcfgr_32 & 0xFFF) >= 4)
		dev->cache.pretrigger	= 1;
	else
		dev->cache.pretrigger	= 0;
}



//*****************************************************************************
static void _voltage_range_compute(dev_data_t* dev)
{
	if (dev->cache.gsc_bcfgr_32 & 0x80000)
		dev->cache.voltage_range	= AI32SSC1M_QUERY_V_RANGE_2_5;
	else
		dev->cache.voltage_range	= AI32SSC1M_QUERY_V_RANGE_10V;
}



//*****************************************************************************
static void _autocal_compute(dev_data_t* dev)
{
	dev->cache.autocal_ms	= 1000;

	if ((dev->cache.gsc_bcfgr_32 & 0xFFF) >= 2)
		dev->cache.autocal_bg	= 1;
	else
		dev->cache.autocal_bg	= 0;
}



//*****************************************************************************
static int _type_compute(dev_data_t* dev)
{
	VADDR_T	_0x40_va;
	u32		_0x40_val;
	VADDR_T	_0x80_va;
	u32		_0x80_val;
	u32		bctlr;
	VADDR_T	bctlr_va;
	VADDR_T	icr_va;
	u32		intcsr;
	VADDR_T	intcsr_va;
	u32		ragr;
	VADDR_T	ragr_va;
	int		ret;
	u16		sdidr;

	sdidr	= os_reg_pci_rx_u16(dev, 0, 0x2E);

	if (sdidr == 0x3431)
	{
		ret	= 0;
	}
	else
	{
		// 18AI32SSC1M: @0x40 != BCR, @0x80 == BCR, RAGR != 0x00010FA0
		_0x40_va	= GSC_VADDR(dev, 0x40);
		_0x80_va	= GSC_VADDR(dev, 0x80);
		bctlr_va	= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_BCTLR));

		_0x40_val	= os_reg_mem_rx_u32(dev, _0x40_va);
		_0x80_val	= os_reg_mem_rx_u32(dev, _0x80_va);
		bctlr		= os_reg_mem_rx_u32(dev, bctlr_va);

		if ((bctlr != _0x40_val) && (bctlr == _0x80_val))
		{
			// This MAY be one of our devices.
			// Reset the device so we can examine the Rate-A Generator Register.
			// Disable interrupts do we don't get an Init Done IRQ.
			icr_va		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_ICR));
			intcsr_va	= PLX_VADDR(dev, 0x68);
			ragr_va		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_RAGR));

			intcsr		= os_reg_mem_rx_u32(dev, intcsr_va);	// Save INTCSR.
			os_reg_mem_tx_u32(dev, intcsr_va, 0);				// Clear INTCSR.
			os_reg_mem_mx_u32(dev, bctlr_va, D15, D15);			// Initialize
			os_time_sleep_ms(10);								// Wait til done.
			os_reg_mem_tx_u32(dev, icr_va, 0);					// Clear the IRQ.
			os_reg_mem_tx_u32(dev, intcsr_va, intcsr);			// Restore INTCSR.

			// Now examine the Rate-A Generator Register.
			ragr	= os_reg_mem_rx_u32(dev, ragr_va);

			if (ragr != 0x00010FA0)
			{
				ret	= 0;	// This IS one of our devices.
			}
			else
			{
				ret	= -ENODEV;	// This is NOT one of our devices.
#if DEV_PCI_ID_SHOW
				printk("This device is not supported by this driver.\n");
#endif
			}
		}
		else
		{
			ret	= -ENODEV;		// This is NOT one of our devices.
#if DEV_PCI_ID_SHOW
			printk("This device is not supported by this driver.\n");
#endif
		}
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	dev_device_create
*
*	Purpose:
*
*		Do everything needed to setup and use the given device.
*
*	Arguments:
*
*		dev		The structure to initialize.
*
*	Returned:
*
*		0		All is well.
*		< 0		An appropriate error status.
*
******************************************************************************/

int dev_device_create(dev_data_t* dev)
{
	static const gsc_bar_maps_t	bar_map	=
	{
		{
			// mem	io	rw
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 0: PLX registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 1: PLX registers, I/O mapped
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RW },	// BAR 2: GSC registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 3: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 4: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO }	// BAR 5: unused
		}
	};

	u32	dma;
	int	ret;

	for (;;)	// A convenience loop.
	{
		// Verify some macro contents.
		ret	= gsc_macro_test_base_name(AI32SSC1M_BASE_NAME);
		if (ret)	break;

		ret	= gsc_macro_test_model();
		if (ret)	break;

		// PCI setup.
		ret	= os_pci_dev_enable(&dev->pci);
		if (ret)	break;

		ret	= os_pci_master_set(&dev->pci);
		if (ret)	break;

		// Control ISR access to the device and data structure.
		ret	= os_spinlock_create(&dev->spinlock);
		if (ret)	break;

		// Control access to the device and data structure.
		ret	= os_sem_create(&dev->sem);
		if (ret)	break;

		// Access the BAR regions.
		ret	= gsc_bar_create(dev, &dev->bar, &bar_map);
		if (ret)	break;

		// Verify that this is a supported device.
		ret	= _type_compute(dev);
		if (ret)	break;

		// Firmware access.
		dev->vaddr.gsc_acar_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_ACAR));
		dev->vaddr.gsc_asiocr_32	= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_ASIOCR));
		dev->vaddr.gsc_bcfgr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_BCFGR));
		dev->vaddr.gsc_bctlr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_BCTLR));
		dev->vaddr.gsc_bstsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_BSTSR));
		dev->vaddr.gsc_bufsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_BUFSR));
		dev->vaddr.gsc_ecdr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_ECDR));
		dev->vaddr.gsc_ibcr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_IBCR));
		dev->vaddr.gsc_ibdr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_IBDR));
		dev->vaddr.gsc_icr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_ICR));
		dev->vaddr.gsc_ptchr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_PTCHR));
		dev->vaddr.gsc_ptclr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_PTCLR));
		dev->vaddr.gsc_ragr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_RAGR));
		dev->vaddr.gsc_rbgr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_RBGR));
		dev->vaddr.gsc_smlwr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_SMLWR));
		dev->vaddr.gsc_smuwr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_SMUWR));
		dev->vaddr.gsc_sscr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AI32SSC1M_GSC_SSCR));

		// Data cache initialization.
		dev->cache.gsc_bcfgr_32			= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcfgr_32);

		dev->cache.fifo_size			= _256K;
		dev->cache.fsamp_max			= 1000000L;
		dev->cache.fsamp_min			= 1L;
		dev->cache.fw_ver				= dev->cache.gsc_bcfgr_32 & 0xFFF;
		dev->cache.initialize_ms		= 3;
		dev->cache.rate_gen_fgen_max	= 1000000L;
		dev->cache.rate_gen_fgen_min	= 1;
		dev->cache.rate_gen_nrate_mask	= 0xFFFF;
		dev->cache.rate_gen_nrate_max	= 0xFFFF;
		dev->cache.rate_gen_nrate_min	= 36;
		dev->cache.rate_gen_qty			= 2;

		_channels_compute(dev);
		_data_width_compute(dev);
		_master_clock_compute(dev);
		_voltage_range_compute(dev);
		_pretrigger_compute(dev);
		_autocal_compute(dev);
		_ext_clk_div_compute(dev);

		// Initialize additional resources.
		ret	= dev_irq_create(dev);
		if (ret)	break;

		ret	= dev_io_create(dev);
		if (ret)	break;

		dma	= GSC_DMA_SEL_STATIC
			| GSC_DMA_CAP_BMDMA_READ
			| GSC_DMA_CAP_DMDMA_READ;
		ret	= gsc_dma_create(dev, dma, dma);
		break;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	dev_device_destroy
*
*	Purpose:
*
*		Do everything needed to release the referenced device.
*
*	Arguments:
*
*		dev		The partial data for the device of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_device_destroy(dev_data_t* dev)
{
	if (dev)
	{
		gsc_dma_destroy(dev);
		dev_io_destroy(dev);
		dev_irq_destroy(dev);
		gsc_bar_destroy(&dev->bar);
		os_sem_destroy(&dev->sem);
		os_spinlock_destroy(&dev->spinlock);
		os_pci_master_clear(&dev->pci);
		os_pci_dev_disable(&dev->pci);
	}
}


