// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/driver/main.h $
// $Rev: 53598 $
// $Date: 2023-08-23 16:05:50 -0500 (Wed, 23 Aug 2023) $

// 18AI32SSC1M: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "18ai32ssc1m.h"

#define	DEV_BAR_SHOW						0
#define	DEV_METRICS_SHOW					0
#define	DEV_PCI_ID_SHOW						0
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_READ
#define	DEV_SUPPORTS_VPD
#define	DEV_SUPPORTS_WAIT
#define	GSC_ALT_DEV_GET(a)					(a)
#define	GSC_ALT_STRUCT_GET(i,d)				(d)
#define	GSC_ALT_STRUCT_T					dev_data_t
#define	GSC_DEVS_PER_BOARD					1

typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// macros *********************************************************************

#define	DEV_MODEL							"18AI32SSC1M"	// Upper case form of the below.
#define	DEV_NAME							"18ai32ssc1m"	// MUST AGREE WITH AI32SSC1M_BASE_NAME

#define	DEV_VERSION							"2.7"			// FOR DEVICE SPECIFIC CODE ONLY!
// 2.7	Updated to support the 6.x series kernel.
//		Added the Autocal Status IOCTL service.
//		Renamed all Auto_Calibration content to Autocal for consistency.
//		Renamed all Auto_Cal_Bg content to Autocal_Bg for consistency.
//		Updated the Input Buffer Clear IOCTL service for consistency.
//		Updated comments for consistency.
//		Updated per PLX code segregation.
//		Updated the DMA code for more modular code organization.
// 2.6	Updated the Initialize IOCTL service to include waiting on the IRQ.
// 2.5	Standardized various IOCTL system log messages.
//		Standardized processing of the initialization IOCTL service.
//		Standardized processing of the autocalibration IOCTL service.
// 2.4	Fixed device type detection bug.
//		Some mods for porting to Windows.
//		Updated initial file content for consistency.
// 2.3	Added support for the 5.x kernel series.
//		Added Endianness support.
// 2.2	Bug fix: was ignoring BAR setup return value.
//		Updated per changes to the OS specific PCI services.
//		Made updates for DEV_SUPPORTS_XXX macro changes.
//		Modified code so open succeeds even if initialization fails.
//		Updated per changes to common I/O code.
//		Added Scan Marker value get/set support.
//		Added Pretrigger Counter access services.
//		Added External Clock Divisor support.
//		Removed the Pretrigger Delay feature as it isn't available to end users.
// 2.1	The BAR code has been updated to include the data under a single structure.
//		The register definitions have been updated.
//		I/O services now pass around an os_mem_t structure instead of a buffer pointer.
//		White space cleanup.
//		Changed the arbitrary wait event callback argument to a void* type.
//		Modified to allow multiple apps to simultaneously access a single device.
//		Correct access to BAR0 and BAR1 (is RO, was RW).
//		Added an infinite I/O timeout option.
//		Corrected the IOCTL code for the register write and mod services.
// 2.0	Updated to use the newer common driver sources.
//		Removed GNU notice from non-Linux specific files.
//		Removed Linux specific content from non-Linux specific source files.
//		White space cleanup.
//		Now using a spinlock rather than enabling and disabling interrupts.
//		Updated gsc_irq_open() and gsc_irq_close().
//		Updated gsc_dma_open() and gsc_dma_close().
//		Enhanced output reported when the DEV_PCI_ID_SHOW option is enabled.
// 1.8	Modified to use newer PLX register header files.
//		Changed from using driver's gsc_kernel_*.h to os_kernel_*.h.
//		Changed from using driver's gsc_sem.c to os_sem.c.
// 1.7	Reduced #include list in driver interface header.
// 1.6	Updated for the 3.x kernel.
//		Corrected a system timer count roll over bug in ioctl.c and device.c.
// 1.5	BAR0 and BAR2 are now the only BAR regions used.
//		Include all common source, though not all are used.
// 1.4	Overhauled IOCTL based local interrupt implementation.
//		Updated per changes to the common source code.
//		Implemented IOCTL based I/O Abort support.
//		Implemented IOCTL based event/interrupt wait support.
//		Updated to use the common gsc_open() and gsc_close() services.
//		Implemented explicit local interrupt support.
//		Implemented support for the common PIO I/O routines.
//		Changed use of DEV_SUPPORTS_READ macro.
//		Changed use of DEV_SUPPORTS_WRITE macro.
//		Added support for Vital Product Data. NOT IMPLEMENTED ON BOARDS.
//		Eliminated the global dev_check_id() routine.
//		Fixed an ISR bug which caused recurring interrupts.
// 1.3	Fixed a bug in the AI32SSC1M_IOCTL_QUERY service.
//		Removed some unused macros from the driver's main.h header.
// 1.2	Fixed a bug in the AI32SSC1M_IOCTL_AIN_BUF_CLEAR service.
//		Updated opena and close services to use the common code base.
//		Added more id information as the device now has a second subdevice id.
//		Modified the AI32SSC1M_IOCTL_INPUT_SYNC service to not wait.
//		Fixed a bug in the query service enumeration.
//		Added support for the pretrigger device option.
// 1.1	Made updates per changes in the Board Configuration Register.
//		Removed spurious printk statements.
//		Made various read support services use the same argument data types.
// 1.0	Initial release of the new driver supporting the basic 18AI32SSC1M.

// I/O services
#define	DEV_IO_STREAM_QTY					(DEV_IO_RX_STREAM_QTY + DEV_IO_TX_STREAM_QTY)
#define	DEV_IO_RX_STREAM_QTY				1
#define	DEV_IO_TX_STREAM_QTY				0

#define	GSC_READ_PIO_WORK_32_BIT

// WAIT services
#define	DEV_WAIT_GSC_ALL					AI32SSC1M_WAIT_GSC_ALL
#define	DEV_WAIT_ALT_ALL					AI32SSC1M_WAIT_ALT_ALL
#define	DEV_WAIT_IO_ALL						AI32SSC1M_WAIT_IO_ALL



// data types *****************************************************************

struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).

	s32					io_mode;			// PIO, DMA, DMDMA
	s32					overflow_check;		// Check overflow when reading?
	s32					pio_threshold;		// Use PIO if samples <= this.
	s32					timeout_s;			// I/O timeout in seconds.
	s32					underflow_check;	// Check underflow when reading?

	// Initialized by I/O service at time of use.

	int					abort;
	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	int					non_blocking;		// Is this non-blocking I/O?

	// Initialized by device specific I/O create code.

	int					bytes_per_sample;	// Sample size in bytes.
	u32					io_reg_offset;		// Offset of device's I/O FIFO.
	VADDR_T				io_reg_vaddr;		// Address of device's I/O FIFO.

	void				(*dev_io_sw_init)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_close)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_open)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	int					(*dev_io_startup)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	long				(*dev_pio_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_bmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_dmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_pio_xfer)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_bmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_dmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

	struct
	{
		u32				abort;			// ...WAIT_IO_XXX_ABORT
		u32				done;			// ...WAIT_IO_XXX_DONE
		u32				error;			// ...WAIT_IO_XXX_ERROR
		u32				timeout;		// ...WAIT_IO_XXX_TIMEOUT
	} wait;

	// Initialize by GSC commone create code.

	os_mem_t			mem;			// I/O buffer.
	os_sem_t			sem;			// Only one Tx or Rx at a time.
};

struct _dev_data_t
{
	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.
	gsc_dev_type_t		board_type;		// Corresponds to basic device type.
	const char*			model;			// Base model number in upper case.
	int					board_index;	// Index of the device being accessed.
	int					users;			// Number of currently active open() requests.
	int					share;			// Were we opened in shared mode?

	gsc_bar_t			bar;			// device register mappings
	gsc_dma_t			dma;			// For DMA based I/O.
	gsc_irq_t			irq;			// For interrut support.
	gsc_vpd_data_t		vpd;			// For serial number and such.
	gsc_wait_node_t*	wait_list;

	struct
	{					// This is for streaming I/O
		dev_io_t		rx;				// Analog Input read
		dev_io_t*		io_streams[DEV_IO_STREAM_QTY];
	} io;

	struct
	{
		VADDR_T			plx_intcsr_32;	// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;	// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;	// DMA Threshold Register

		VADDR_T			gsc_acar_32;	// Active Channel Assignment Register
		VADDR_T			gsc_asiocr_32;	// Auxiliary Sync I/O Control Register
		VADDR_T			gsc_bcfgr_32;	// Board Configuration Register
		VADDR_T			gsc_bctlr_32;	// Board Control Register
		VADDR_T			gsc_bstsr_32;	// Burst Size Register
		VADDR_T			gsc_bufsr_32;	// Buffer Size Register
		VADDR_T			gsc_ecdr_32;	// External Clock Divisor Register
		VADDR_T			gsc_ibcr_32;	// Input Buffer Control Register
		VADDR_T			gsc_ibdr_32;	// Input Buffer Data Register
		VADDR_T			gsc_icr_32;		// Interrupt Control Register
		VADDR_T			gsc_ptchr_32;	// Pretrigger Counter High Register
		VADDR_T			gsc_ptclr_32;	// Pretrigger Counter Low Register
		VADDR_T			gsc_ragr_32;	// Rate-A Generator Register
		VADDR_T			gsc_rbgr_32;	// Rate-B Generator Register
		VADDR_T			gsc_smlwr_32;	// Scan Marker Lower Word Register
		VADDR_T			gsc_smuwr_32;	// Scan Marker Upper Word Register
		VADDR_T			gsc_sscr_32;	// Scan & Sync Control Register
	} vaddr;

	struct
	{
		u32				gsc_bcfgr_32;		// Board Configuration Register

		s32				autocal_bg;			// Is Back Ground Autocal supported?
		s32				autocal_ms;			// Maximum ms for autocalibrate

		s32				channels_max;		// Maximum channels supported by model.
		s32				channel_qty;		// The number of channels on the device.

		s32				data_width;			// The native data width.

		s32				ext_clk_div;		// Is the External Clock Divisor available?

		s32				fifo_size;			// Size of FIFO - not the fill level.
		s32				fsamp_max;			// The maximum Fsamp rate per channel.
		s32				fsamp_min;			// The minimum Fsamp rate per channel.
		s32				fw_ver;				// The firmware version.

		s32				initialize_ms;		// Maximum ms for initialize

		s32				master_clock;		// Master clock frequency

		s32				pretrigger;			// The Pretrigger feature.

		s32				rate_gen_fgen_max;	// Rate Generator maximum output rate.
		s32				rate_gen_fgen_min;	// Rate Generator minimum output rate.
		s32				rate_gen_nrate_mask;// Mask of valid Nrate bits.
		s32				rate_gen_nrate_max;	// Minimum Nrate value.
		s32				rate_gen_nrate_min;	// Maximum Nrate value.
		s32				rate_gen_qty;		// The number of rate generators on device.

		s32				voltage_range;		// 10V/5V or 2.5V/1.25V

	} cache;
};



// prototypes *****************************************************************

void		dev_io_close(dev_data_t* dev);
int			dev_io_create(dev_data_t* dev);
void		dev_io_destroy(dev_data_t* dev);
int			dev_io_open(dev_data_t* dev);
dev_io_t*	dev_io_read_select(dev_data_t* dev, size_t count);
int			dev_irq_create(dev_data_t* dev);
void		dev_irq_destroy(dev_data_t* dev);
int			dev_read_create(dev_data_t* dev, dev_io_t* io);

int			initialize_ioctl(dev_data_t* dev, void* arg);



#endif
