// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/samples/rxrate/perform.c $
// $Rev: 53592 $
// $Date: 2023-08-23 15:53:54 -0500 (Wed, 23 Aug 2023) $

// 18AI32SSC1M: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M		(1024 * 1024)	// size of driver's transfer buffer



// variables ******************************************************************

static	u8	_data[_1M];



//*****************************************************************************
static int _config(const args_t* args)
{
	int	errs	= 0;
	s32	fsamp;

	gsc_label("Configuration");
	printf("\n");
	gsc_label_level_inc();

	errs	+= ai32ssc1m_initialize			(args->fd, -1, 1);
	errs	+= ai32ssc1m_query				(args->fd, -1, 1, AI32SSC1M_QUERY_FSAMP_MAX, &fsamp);
	errs	+= ai32ssc1m_config_ai			(args->fd, -1, 1, fsamp);
	errs	+= ai32ssc1m_rx_io_mode			(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= ai32ssc1m_rx_io_overflow		(args->fd, -1, 1, AI32SSC1M_IO_OVERFLOW_IGNORE, NULL);
	errs	+= ai32ssc1m_rx_io_underflow	(args->fd, -1, 1, AI32SSC1M_IO_UNDERFLOW_IGNORE, NULL);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _rate_test(const args_t* args, long get)
{
	u32				avail;
	struct timeval	begin;
	int				errs	= 0;
	long			got;
	long long		limit	= (long long) args->rx_mb * 1000000L;
	int				i;
	struct timeval	minimum;
	struct timeval	now;
	u32				reg;
	long long		sps;
	long long		total	= 0;
	long			us		= 1000000L * args->seconds;

	gsc_label("Reading");

	gettimeofday(&begin, NULL);
	minimum.tv_sec	= begin.tv_sec + args->seconds;
	minimum.tv_usec	= begin.tv_usec;

	for (;;)
	{
		i	= ai32ssc1m_reg_read(args->fd, -1, 0, AI32SSC1M_GSC_BUFSR, &reg);

		if (i < 0)
		{
			errs++;
			printf("FAIL <--- (ai32ssc1m_reg_read, status %d)\n", i);
			break;
		}

		avail	= reg & 0x7FFFF;

		if (avail < 8192)
		{
			os_sleep_ms(1);
			continue;
		}

		got	= ai32ssc1m_read(args->fd, _data, get);

		if (got < 0)
		{
			errs++;
			printf("FAIL <--- (ai32ssc1m_read");
			printf(", total so far %lld", total);
			printf(", requested %ld bytes", (long) get);
			printf(", errno %d)\n", (int) -got);
			break;
		}

		gettimeofday(&now, NULL);
		total	+= got;

		if ((total >= limit) && (now.tv_sec > minimum.tv_sec))
		{
			us	= (now.tv_sec - begin.tv_sec) * 1000000L
				+ (now.tv_usec - begin.tv_usec);
			sps	= (total * 10000000L / 4 / us + 5) / 10;

			printf(	"PASS  (");
			gsc_label_long_comma(total);
			printf(" Bytes, %ld.%06ld Seconds, ", us / 1000000, us % 1000000);
			gsc_label_long_comma(sps);
			printf(" S/S)\n");
			break;
		}
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int		errs	= 0;
	long	get		= sizeof(_data);

	errs	+= _config(args);

	gsc_label ("Read Rate");
	printf("(%d second minimum, %ld MB minimum)\n", args->seconds, (long) args->rx_mb);
	gsc_label_level_inc();

	errs	+= ai32ssc1m_ain_buf_clear		(args->fd, -1, 1, 1);
	errs	+= ai32ssc1m_ain_buf_overflow	(args->fd, -1, 1, AI32SSC1M_AIN_BUF_OVERFLOW_CLEAR, NULL);
	errs	+= ai32ssc1m_ain_buf_underflow	(args->fd, -1, 1, AI32SSC1M_AIN_BUF_UNDERFLOW_CLEAR, NULL);

	errs	+= _rate_test(args, get);

	errs	+= ai32ssc1m_ain_buf_overflow	(args->fd, -1, 1, -1, NULL);
	errs	+= ai32ssc1m_ain_buf_underflow	(args->fd, -1, 1, -1, NULL);

	gsc_label_level_dec();
	return(errs);
}



