// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/samples/sbtest/burst_busy.c $
// $Rev: 53596 $
// $Date: 2023-08-23 15:57:48 -0500 (Wed, 23 Aug 2023) $

// 18AI32SSC1M: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		// Produce a short sync operation.

		{				// Burst size: small
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_BURST_SIZE,
			/* arg		*/	5,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Burst source = BCR.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_BURST_SYNC,
			/* arg		*/	AI32SSC1M_BURST_SYNC_BCR,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Scan channels 0-1
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_CHAN_ACTIVE,
			/* arg		*/	AI32SSC1M_CHAN_ACTIVE_0_1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Nrate: fast
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_RAG_NRATE,
			/* arg		*/	40,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// ADC source = Rate A Generator
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_SAMP_CLK_SRC,
			/* arg		*/	AI32SSC1M_SAMP_CLK_SRC_RAG,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Enable Rate-A Generator
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_RAG_ENABLE,
			/* arg		*/	AI32SSC1M_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Enable ADC operation.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_CLOCK_ENABLE,
			/* arg		*/	AI32SSC1M_CLOCK_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Perform a Burst Sync operation.
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSC1M_GSC_BCTLR,
			/* mask		*/	0x1000,
			/* value	*/	0x1000
		},
		{				// Wait for operation activity.
			/* service	*/	SERVICE_SLEEP,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Verify that the sync operation is done.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSC1M_GSC_BCTLR,
			/* mask		*/	0x1000,
			/* value	*/	0x0000
		},
		{				// Verify that the burst operation is done.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSC1M_GSC_SSCR,
			/* mask		*/	0x80,
			/* value	*/	0x00
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		// Produce a long sync operation.

		{				// Burst size: large
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_BURST_SIZE,
			/* arg		*/	0xFFFFF,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Burst source = BCR.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_BURST_SYNC,
			/* arg		*/	AI32SSC1M_BURST_SYNC_BCR,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Scan channels 0-1
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_CHAN_ACTIVE,
			/* arg		*/	AI32SSC1M_CHAN_ACTIVE_0_1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Nrate: slow
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_RAG_NRATE,
			/* arg		*/	0xFFFF,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// ADC source = Rate A Generator
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_SAMP_CLK_SRC,
			/* arg		*/	AI32SSC1M_SAMP_CLK_SRC_RAG,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Enable Rate-A Generator
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_RAG_ENABLE,
			/* arg		*/	AI32SSC1M_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Enable ADC operation.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_CLOCK_ENABLE,
			/* arg		*/	AI32SSC1M_CLOCK_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Perform a Burst Sync operation.
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSC1M_GSC_BCTLR,
			/* mask		*/	0x1000,
			/* value	*/	0x1000
		},
		{				// Wait for operation activity.
			/* service	*/	SERVICE_SLEEP,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Verify that the sync operation is done.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSC1M_GSC_BCTLR,
			/* mask		*/	0x1000,
			/* value	*/	0x0000
		},
		{				// Verify that the sync operation is still active.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSC1M_GSC_SSCR,
			/* mask		*/	0x80,
			/* value	*/	0x80
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;
	s32	sts;

	errs	+= ai32ssc1m_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__,list_1);

	if (errs == 0)
	{
		// Verify that the operation is complete.
		ret		= ai32ssc1m_ioctl(fd, AI32SSC1M_IOCTL_BURST_BUSY, &sts);
		errs	= ret ? 1 : 0;

		if (ret)
		{
			errs++;
		}
		else if (sts != AI32SSC1M_BURST_BUSY_IDLE)
		{
			errs++;
			printf(	"FAIL <---  (%d. expected idle, but still active)\n",
					__LINE__);
		}
	}

	errs	+= service_ioctl_set_reg_list(fd, __LINE__,list_2);

	if (errs == 0)
	{
		// Verify that the operation is still active.
		ret		= ai32ssc1m_ioctl(fd, AI32SSC1M_IOCTL_BURST_BUSY, &sts);
		errs	= ret ? 1 : 0;

		if (ret)
		{
			errs++;
		}
		else if (sts != AI32SSC1M_BURST_BUSY_ACTIVE)
		{
			errs++;
			printf(	"FAIL <---  (%d. expected active, but is idle)\n",
					__LINE__);
		}
	}

	errs	+= ai32ssc1m_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	burst_busy_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AI32SSC1M_IOCTL_BURST_BUSY.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int burst_busy_test(int fd)
{
	int	errs	= 0;

	gsc_label("AI32SSC1M_IOCTL_BURST_BUSY");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


