// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/samples/sbtest/burst_size.c $
// $Rev: 53596 $
// $Date: 2023-08-23 15:57:48 -0500 (Wed, 23 Aug 2023) $

// 18AI32SSC1M: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const s32	args[]	=
	{
		0x00000,
		0x00001,
		0x00002,
		0x00004,
		0x00008,
		0x00010,
		0x00020,
		0x00040,
		0x00080,
		0x00100,
		0x00200,
		0x00400,
		0x00800,
		0x01000,
		0x02000,
		0x04000,
		0x08000,
		0x10000,
		0x20000,
		0x30000,
		0x40000,
		0x00000,

		0xFFFFF,
		0xFFFFE,
		0xFFFFD,
		0xFFFFB,
		0xFFFF7,
		0xFFFEF,
		0xFFFDF,
		0xFFFBF,
		0xFFF7F,
		0xFFEFF,
		0xFFDFF,
		0xFFBFF,
		0xFF7FF,
		0xFEFFF,
		0xFDFFF,
		0xFBFFF,
		0xF7FFF,
		0xEFFFF,
		0xDFFFF,
		0xBFFFF,
		0x7FFFF,
		0xFFFFF,

		0x00001,
		-1
	};

	int	errs	= 0;

	errs	+= ai32ssc1m_initialize(fd, -1, 0);
	errs	+= service_test_set_reg_reg_get(
				fd,
				__LINE__,
				AI32SSC1M_IOCTL_BURST_SIZE, args,
				AI32SSC1M_GSC_BSTSR, 19, 0);
	errs	+= ai32ssc1m_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	service_data_t	list[]	=
	{
		{	// 0			// Initialize the device.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 1			// Set the burst size.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_BURST_SIZE,
			/* arg		*/	0,	// SET THIS FOR EACH LOOP ITERATION.
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 2			// Burst Source: BCR
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_BURST_SYNC,
			/* arg		*/	AI32SSC1M_BURST_SYNC_BCR,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 3			// Scan a single channel only.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_CHAN_ACTIVE,
			/* arg		*/	AI32SSC1M_CHAN_ACTIVE_SINGLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 4			// Use channel zero.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_CHAN_SINGLE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 5		// Nrate: fast
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_RAG_NRATE,
			/* arg		*/	40,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 6			// Scan Clock: Rate-A Generator
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_SAMP_CLK_SRC,
			/* arg		*/	AI32SSC1M_SAMP_CLK_SRC_RAG,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 7			// Enable Rate-A Generator
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_RAG_ENABLE,
			/* arg		*/	AI32SSC1M_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 8			// Enable clocking.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_CLOCK_ENABLE,
			/* arg		*/	AI32SSC1M_CLOCK_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 9			// Perform an Input Sync operation.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_INPUT_SYNC,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 10			// Wait for possible data reception.
			/* service	*/	SERVICE_SLEEP,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 11			// Verify the amount of data.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSC1M_GSC_BUFSR,
			/* mask		*/	0x3FFFF,
			/* value	*/	0	// SET IN EACH LOOP ITERATION
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	i;

	errs	+= ai32ssc1m_initialize(fd, -1, 0);

	// We're only testing the level accuracy for a small range.

	for (i = 5; (errs == 0) && (i < 10); i++)
	{
		list[1].arg		= 111 * i;
		list[11].value	= 111 * i;
		errs			+= service_ioctl_set_reg_list(fd, __LINE__, list);
	}

	errs	+= ai32ssc1m_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	burst_size_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AI32SSC1M_IOCTL_BURST_SIZE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int burst_size_test(int fd)
{
	int	errs	= 0;

	gsc_label("AI32SSC1M_IOCTL_BURST_SIZE");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


