// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/samples/sbtest/chan_active.c $
// $Rev: 53596 $
// $Date: 2023-08-23 15:57:48 -0500 (Wed, 23 Aug 2023) $

// 18AI32SSC1M: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const s32	args[]	=
	{
		AI32SSC1M_CHAN_ACTIVE_SINGLE,
		AI32SSC1M_CHAN_ACTIVE_0_1,
		AI32SSC1M_CHAN_ACTIVE_0_3,
		AI32SSC1M_CHAN_ACTIVE_0_7,
		AI32SSC1M_CHAN_ACTIVE_0_15,
		AI32SSC1M_CHAN_ACTIVE_RANGE,
		-1
	};

	int	errs	= 0;

	errs	+= ai32ssc1m_initialize(fd, -1, 0);
	errs	+= service_test_set_reg_reg_get(
				fd,
				__LINE__,
				AI32SSC1M_IOCTL_CHAN_ACTIVE, args,
				AI32SSC1M_GSC_SSCR, 2, 0);
	errs	+= ai32ssc1m_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_sub_test(int fd, s32 option, int samples, int first, int last)
{
	const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Select option
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_CHAN_ACTIVE,
			/* arg		*/	option,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// First Channel
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_CHAN_FIRST,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Last Channel
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_CHAN_LAST,
			/* arg		*/	last,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// First Channel
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_CHAN_FIRST,
			/* arg		*/	first,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// ADC source = BCR Control Bit
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_SAMP_CLK_SRC,
			/* arg		*/	AI32SSC1M_SAMP_CLK_SRC_BCR,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Enable ADC operation.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_CLOCK_ENABLE,
			/* arg		*/	AI32SSC1M_CLOCK_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Perform an Input Sync operation.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_INPUT_SYNC,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Check the results
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	AI32SSC1M_GSC_BUFSR,
			/* mask		*/	0x7FFFF,
			/* value	*/	samples
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	errs	= service_ioctl_set_reg_list(fd, __LINE__, list);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	int errs	= 0;

	errs	+= ai32ssc1m_initialize(fd, -1, 0);

	errs	+= _function_sub_test(fd, AI32SSC1M_CHAN_ACTIVE_SINGLE,	1,	0,	0	);
	errs	+= _function_sub_test(fd, AI32SSC1M_CHAN_ACTIVE_0_1,	2,	0,	0	);
	errs	+= _function_sub_test(fd, AI32SSC1M_CHAN_ACTIVE_0_3,	4,	0,	0	);
	errs	+= _function_sub_test(fd, AI32SSC1M_CHAN_ACTIVE_0_7,	8,	0,	0	);
	errs	+= _function_sub_test(fd, AI32SSC1M_CHAN_ACTIVE_0_15,	16,	0,	0	);
	errs	+= _function_sub_test(fd, AI32SSC1M_CHAN_ACTIVE_RANGE,	2,	13,	14	);
	errs	+= _function_sub_test(fd, AI32SSC1M_CHAN_ACTIVE_RANGE,	6,	1,	6	);
	errs	+= _function_sub_test(fd, AI32SSC1M_CHAN_ACTIVE_RANGE,	10,	3,	12	);

	errs	+= ai32ssc1m_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	chan_active_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AI32SSC1M_IOCTL_CHAN_ACTIVE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int chan_active_test(int fd)
{
	int	errs	= 0;

	gsc_label("AI32SSC1M_IOCTL_CHAN_ACTIVE");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


