// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/samples/sbtest/data_packing.c $
// $Rev: 53596 $
// $Date: 2023-08-23 15:57:48 -0500 (Wed, 23 Aug 2023) $

// 18AI32SSC1M: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const s32	args[]	=
	{
		AI32SSC1M_DATA_PACKING_DISABLE,
		AI32SSC1M_DATA_PACKING_ENABLE,
		-1
	};

	int	errs	= 0;

	errs	+= ai32ssc1m_initialize(fd, -1, 0);
	errs	+= service_test_set_reg_reg_get(
				fd,
				__LINE__,
				AI32SSC1M_IOCTL_DATA_PACKING, args,
				AI32SSC1M_GSC_BCTLR, 18, 18);
	errs	+= ai32ssc1m_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list[]	=
	{
		// Test 1: unpacked, 8 samples, fifo size = 8

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Scan channels 0-7
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_CHAN_ACTIVE,
			/* arg		*/	AI32SSC1M_CHAN_ACTIVE_0_7,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// ADC source = BCR Control Bit
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_SAMP_CLK_SRC,
			/* arg		*/	AI32SSC1M_SAMP_CLK_SRC_BCR,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Enable ADC operation.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_CLOCK_ENABLE,
			/* arg		*/	AI32SSC1M_CLOCK_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_DATA_PACKING,
			/* arg		*/	AI32SSC1M_DATA_PACKING_DISABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Perform an Input Sync operation.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_INPUT_SYNC,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Check the results
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	AI32SSC1M_GSC_BUFSR,
			/* mask		*/	0x7FFFF,
			/* value	*/	0x00008
		},

		// Test 2: packed, 8 samples, fifo size = 4

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Scan channels 0-7
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_CHAN_ACTIVE,
			/* arg		*/	AI32SSC1M_CHAN_ACTIVE_0_7,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// ADC source = BCR Control Bit
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_SAMP_CLK_SRC,
			/* arg		*/	AI32SSC1M_SAMP_CLK_SRC_BCR,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Enable ADC operation.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_CLOCK_ENABLE,
			/* arg		*/	AI32SSC1M_CLOCK_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_DATA_PACKING,
			/* arg		*/	AI32SSC1M_DATA_PACKING_ENABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Disable the scan marker.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_SCAN_MARKER,
			/* arg		*/	AI32SSC1M_SCAN_MARKER_DISABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Perform an Input Sync operation.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_INPUT_SYNC,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Check the results
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	AI32SSC1M_GSC_BUFSR,
			/* mask		*/	0x7FFFF,
			/* value	*/	0x00004
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= ai32ssc1m_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);

	errs	+= ai32ssc1m_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	data_packing_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AI32SSC1M_IOCTL_DATA_PACKING.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int data_packing_test(int fd)
{
	int	errs	= 0;

	gsc_label("AI32SSC1M_IOCTL_DATA_PACKING");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


