// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/samples/sbtest/data_width.c $
// $Rev: 53596 $
// $Date: 2023-08-23 15:57:48 -0500 (Wed, 23 Aug 2023) $

// 18AI32SSC1M: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const s32	args[]	=
	{
		AI32SSC1M_DATA_WIDTH_16,
		AI32SSC1M_DATA_WIDTH_18,
		-1
	};

	int	errs	= 0;

	errs	+= ai32ssc1m_initialize(fd, -1, 0);
	errs	+= service_test_set_reg_reg_get(
				fd,
				__LINE__,
				AI32SSC1M_IOCTL_DATA_WIDTH, args,
				AI32SSC1M_GSC_IBCR, 20, 20);
	errs	+= ai32ssc1m_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list[]	=
	{
		// 16-bit data

		{					// Initialize the device
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Set the input buffer threshold level very low.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_AIN_BUF_THR_LVL,
			/* arg		*/	0x00001,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Select 16-bit mode
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_DATA_WIDTH,
			/* arg		*/	AI32SSC1M_DATA_WIDTH_16,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Select _Vref mode
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_AIN_MODE,
			/* arg		*/	AI32SSC1M_AIN_MODE_VREF,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Select Rate-A Generator operation.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_SAMP_CLK_SRC,
			/* arg		*/	AI32SSC1M_SAMP_CLK_SRC_RAG,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_RAG_ENABLE,
			/* arg		*/	AI32SSC1M_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Enable data clocking.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_CLOCK_ENABLE,
			/* arg		*/	AI32SSC1M_CLOCK_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Wait a moment.
			/* service	*/	SERVICE_SLEEP,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Verify that the threshold flag is set.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSC1M_GSC_IBCR,
			/* mask		*/	0x80000,
			/* value	*/	0x80000
		},
		{					// Verify bits 16 and 17.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	AI32SSC1M_GSC_IBDR,
			/* mask		*/	0x30000,
			/* value	*/	0x00000
		},

		// 18-bit data

		{					// Initialize the device
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Set the input buffer threshold level very low.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_AIN_BUF_THR_LVL,
			/* arg		*/	0x00001,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Select 18-bit mode
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_DATA_WIDTH,
			/* arg		*/	AI32SSC1M_DATA_WIDTH_18,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Select _Vref mode
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_AIN_MODE,
			/* arg		*/	AI32SSC1M_AIN_MODE_VREF,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Select Rate-A Generator operation.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_SAMP_CLK_SRC,
			/* arg		*/	AI32SSC1M_SAMP_CLK_SRC_RAG,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_RAG_ENABLE,
			/* arg		*/	AI32SSC1M_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Enable data clocking.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSC1M_IOCTL_CLOCK_ENABLE,
			/* arg		*/	AI32SSC1M_CLOCK_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Wait a moment.
			/* service	*/	SERVICE_SLEEP,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Verify that the threshold flag is set.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSC1M_GSC_IBCR,
			/* mask		*/	0x80000,
			/* value	*/	0x80000
		},
		{					// Verify bits 16 and 17.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	AI32SSC1M_GSC_IBDR,
			/* mask		*/	0x30000,
			/* value	*/	0x30000
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= ai32ssc1m_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);

	errs	+= ai32ssc1m_initialize(fd, -1, 0);

	return(errs);
}



/******************************************************************************
*
*	Function:	data_width_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AI32SSC1M_IOCTL_DATA_WIDTH.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int data_width_test(int fd)
{
	int	errs	= 0;
	s32	width;

	gsc_label("AI32SSC1M_IOCTL_DATA_WIDTH");
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_DATA_WIDTH, &width);

	if (errs)
	{
	}
	else if (width == 16)
	{
		printf("SKIPPED  (Not supported in this firmware.)\n");
	}
	else
	{
		errs	+= _service_test(fd);
		errs	+= _function_test(fd);

		if (errs == 0)
			printf("PASS\n");
	}

	return(errs);
}


