// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/samples/sbtest/main.h $
// $Rev: 53596 $
// $Date: 2023-08-23 15:57:48 -0500 (Wed, 23 Aug 2023) $

// 18AI32SSC1M: Sample Application: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include <errno.h>
#include <memory.h>
#include <semaphore.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "18ai32ssc1m_main.h"



// macros *********************************************************************

#define	SIZEOF_ARRAY(a)			(sizeof((a)) / sizeof((a)[0]))



// data types *****************************************************************

typedef struct
{
	// Application Settings

	s32	continuous;		// > 0 = ignore errors, < 0 = stop on errors
	s32	index;			// device index
	s32	minute_limit;	// for continuous testing
	int	qty;			// Number of devices detected.
	s32	test_limit;		// for continuous testing

	int	fd;				// File descriptor for device to access.

	// Device Settings

} args_t;

typedef enum
{
	SERVICE_END_LIST,	// Ends list of service_data_t structures.
	SERVICE_NORMAL,
	SERVICE_REG_MOD,
	SERVICE_REG_READ,
	SERVICE_REG_SHOW,
	SERVICE_REG_TEST,
	SERVICE_SLEEP,		// arg = # of seconds to sleep
	SERVICE_IOCTL_GET,
	SERVICE_IOCTL_SET
} service_t;

typedef struct
{
	service_t		service;
	unsigned long	cmd;	// IOCTL code
	s32				arg;	// The IOCTL data argument.
	u32				reg;	// The register to access. Use -1 to ignore.
	u32				mask;	// The register bits of interest.
	u32				value;	// The value expected for the bits of interest.
} service_data_t;



// prototypes *****************************************************************

int	ain_buf_clear_test(int fd);		// AI32SSC1M_IOCTL_AIN_BUF_CLEAR
int	ain_buf_level_test(int fd);		// AI32SSC1M_IOCTL_AIN_BUF_LEVEL
int	ain_buf_overflow_test(int fd);	// AI32SSC1M_IOCTL_AIN_BUF_OVERFLOW
int	ain_buf_thr_lvl_test(int fd);	// AI32SSC1M_IOCTL_AIN_BUF_THR_LVL
int	ain_buf_thr_sts_test(int fd);	// AI32SSC1M_IOCTL_AIN_BUF_THR_STS
int	ain_buf_underflow_test(int fd);	// AI32SSC1M_IOCTL_AIN_BUF_UNDERFLOW
int	ain_mode_test(int fd);			// AI32SSC1M_IOCTL_AIN_MODE
int	ain_range_test(int fd);			// AI32SSC1M_IOCTL_AIN_RANGE
int	autocal_test(int fd);			// AI32SSC1M_IOCTL_AUTOCAL
int	autocal_bg_test(int fd);		// AI32SSC1M_IOCTL_AUTOCAL_BG
int	autocal_status_test(int fd);	// AI32SSC1M_IOCTL_AUTOCAL_STATUS
int	aux_clk_mode_test(int fd);		// AI32SSC1M_IOCTL_AUX_CLK_MODE
int	aux_in_pol_test(int fd);		// AI32SSC1M_IOCTL_AUX_IN_POL
int	aux_noise_test(int fd);			// AI32SSC1M_IOCTL_AUX_NOISE
int	aux_out_pol_test(int fd);		// AI32SSC1M_IOCTL_AUX_OUT_POL
int	aux_sync_mode_test(int fd);		// AI32SSC1M_IOCTL_AUX_SYNC_MODE

int	burst_busy_test(int fd);		// AI32SSC1M_IOCTL_BURST_BUSY
int	burst_size_test(int fd);		// AI32SSC1M_IOCTL_BURST_SIZE
int	burst_sync_test(int fd);		// AI32SSC1M_IOCTL_BURST_SYNC

int	chan_active_test(int fd);		// AI32SSC1M_IOCTL_CHAN_ACTIVE
int	chan_first_test(int fd);		// AI32SSC1M_IOCTL_CHAN_FIRST
int	chan_last_test(int fd);			// AI32SSC1M_IOCTL_CHAN_LAST
int	chan_single_test(int fd);		// AI32SSC1M_IOCTL_CHAN_SINGLE
int	clock_enable_test(int fd);		// AI32SSC1M_IOCTL_CLOCK_ENABLE

int	data_format_test(int fd);		// AI32SSC1M_IOCTL_DATA_FORMAT
int	data_packing_test(int fd);		// AI32SSC1M_IOCTL_DATA_PACKING
int	data_width_test(int fd);		// AI32SSC1M_IOCTL_DATA_WIDTH

int	ext_clk_div_ena_test(int fd);	// AI32SSC1M_IOCTL_EXT_CLK_DIV_ENA
int	ext_clk_div_val_test(int fd);	// AI32SSC1M_IOCTL_EXT_CLK_DIV_VAL

int	initialize_test(int fd);		// AI32SSC1M_IOCTL_INITIALIZE
int	input_sync_test(int fd);		// AI32SSC1M_IOCTL_INPUT_SYNC
int	io_inv_test(int fd);			// AI32SSC1M_IOCTL_IO_INV
int	irq0_sel_test(int fd);			// AI32SSC1M_IOCTL_IRQ0_SEL
int	irq1_sel_test(int fd);			// AI32SSC1M_IOCTL_IRQ1_SEL

int	perform_tests(const args_t* args);
int	pretrig_count_test(int fd);		// AI32SSC1M_IOCTL_PRETRIG_COUNT
int	pretrig_latch_test(int fd);		// AI32SSC1M_IOCTL_PRETRIG_LATCH
int	pretrig_test(int fd);			// AI32SSC1M_IOCTL_PRETRIG

int	query_test(int fd);				// AI32SSC1M_IOCTL_QUERY

int	rag_enable_test(int fd);		// AI32SSC1M_IOCTL_RAG_ENABLE
int	rag_nrate_test(int fd);			// AI32SSC1M_IOCTL_RAG_NRATE
int	rbg_clk_src_test(int fd);		// AI32SSC1M_IOCTL_RBG_CLK_SRC
int	rbg_enable_test(int fd);		// AI32SSC1M_IOCTL_RBG_ENABLE
int	rbg_nrate_test(int fd);			// AI32SSC1M_IOCTL_RBG_NRATE
int	reg_mod_test(int fd);			// AI32SSC1M_IOCTL_REG_MOD
int	reg_read_test(int fd);			// AI32SSC1M_IOCTL_REG_READ
int	reg_write_test(int fd);			// AI32SSC1M_IOCTL_REG_WRITE
int	rx_io_abort_test(int fd);		// AI32SSC1M_IOCTL_RX_IO_ABORT
int	rx_io_mode_test(int fd);		// AI32SSC1M_IOCTL_RX_IO_MODE
int	rx_io_overflow_test(int fd);	// AI32SSC1M_IOCTL_RX_IO_OVERFLOW
int	rx_io_timeout_test(int fd);		// AI32SSC1M_IOCTL_RX_IO_TIMEOUT
int	rx_io_underflow_test(int fd);	// AI32SSC1M_IOCTL_RX_IO_UNDERFLOW

int	samp_clk_src_test(int fd);		// AI32SSC1M_IOCTL_SAMP_CLK_SRC
int	scan_marker_get_test(int fd);	// AI32SSC1M_IOCTL_SCAN_MARKER_GET
int	scan_marker_set_test(int fd);	// AI32SSC1M_IOCTL_SCAN_MARKER_SET
int	scan_marker_test(int fd);		// AI32SSC1M_IOCTL_SCAN_MARKER

int	service_ioctl_reg_get_list	(int fd, int line, const service_data_t* list);
int	service_ioctl_set_get_list	(int fd, int line, const service_data_t* list);
int	service_ioctl_set_reg_list	(int fd, int line, const service_data_t* list);
int service_test_set_get		(int fd, int line, s32 cmd, const s32* args);
int	service_test_set_reg_reg_get(int fd, int line, s32 cmd, const s32* args, s32 reg, int begin, int end);

int	vpd_read_test(int fd);			// AI32SSC1M_IOCTL_VPD_READ

int	wait_cancel_test(int fd);		// AI32SSC1M_IOCTL_WAIT_CANCEL
int	wait_event_test(int fd);		// AI32SSC1M_IOCTL_WAIT_EVENT
int	wait_status_test(int fd);		// AI32SSC1M_IOCTL_WAIT_STATUS



#endif
