// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/samples/sbtest/reg_mod.c $
// $Rev: 53596 $
// $Date: 2023-08-23 15:57:48 -0500 (Wed, 23 Aug 2023) $

// 18AI32SSC1M: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const struct
	{
		u32	reg;
		u32	mask;
	} list[]	=
	{
		// reg					mask
		{ AI32SSC1M_GSC_BCTLR,	0x00040877	},
		{ AI32SSC1M_GSC_SSCR,	0x00000F38	},
	};

	u32			bit;
	int			errs	= 0;
	int			i;
	int			j;
	gsc_reg_t	parm;
	u32			value;

	for (i = 0; (errs == 0) && (i < SIZEOF_ARRAY(list)); i++)
	{

		for (bit = 0x1; bit; bit <<= 1)
		{
			if ((list[i].mask & bit) == 0)
				continue;

			// Read the register value so we can restore it later.
			parm.reg	= list[i].reg;
			j			= ai32ssc1m_ioctl(fd, AI32SSC1M_IOCTL_REG_READ, (void*) &parm);
			value		= parm.value;

			if (j < 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, ai32ssc1m_ioctl() failure, status %d)\n",
						__LINE__,
						i,
						j);
				break;
			}

			// Set the bit low.
			parm.value	= 0;
			parm.mask	= bit;
			j			= ai32ssc1m_ioctl(fd, AI32SSC1M_IOCTL_REG_MOD, (void*) &parm);

			if (j < 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, ai32ssc1m_ioctl() failure, status %d)\n",
						__LINE__,
						i,
						j);
				break;
			}

			// Now verify that the bit is low.
			j	= ai32ssc1m_ioctl(fd, AI32SSC1M_IOCTL_REG_READ, (void*) &parm);

			if (j < 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, ai32ssc1m_ioctl() failure, status %d)\n",
						__LINE__,
						i,
						j);
				break;
			}

			if (parm.value & bit)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, bit not low, mask 0x%lX, got 0x%lX)\n",
						__LINE__,
						i,
						(long) bit,
						(long) parm.value);
				break;
			}

			// Now restore the register to its previous value.
			parm.value	= value;
			j			= ai32ssc1m_ioctl(fd, AI32SSC1M_IOCTL_REG_WRITE, (void*) &parm);

			if (j < 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, ai32ssc1m_ioctl() failure, status %d)\n",
						__LINE__,
						i,
						j);
				break;
			}

			// Set the bit high.
			parm.value	= bit;
			parm.mask	= bit;
			j			= ai32ssc1m_ioctl(fd, AI32SSC1M_IOCTL_REG_MOD, (void*) &parm);

			if (j < 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, ai32ssc1m_ioctl() failure, status %d)\n",
						__LINE__,
						i,
						j);
				break;
			}

			// Now verify that the bit is high.
			j	= ai32ssc1m_ioctl(fd, AI32SSC1M_IOCTL_REG_READ, (void*) &parm);

			if (j < 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, ai32ssc1m_ioctl() failure, status %d)\n",
						__LINE__,
						i,
						j);
				break;
			}

			if ((parm.value & bit) == 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, bit not high, mask 0x%lX, got 0x%lX)\n",
						__LINE__,
						i,
						(long) bit,
						(long) parm.value);
				break;
			}

			// Now restore the register to its previous value.
			parm.value	= value;
			j			= ai32ssc1m_ioctl(fd, AI32SSC1M_IOCTL_REG_WRITE, (void*) &parm);

			if (j < 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, ai32ssc1m_ioctl() failure, status %d)\n",
						__LINE__,
						i,
						j);
				break;
			}
		}
	}

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	return(0);
}



/******************************************************************************
*
*	Function:	reg_mod_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AI32SSC1M_IOCTL_REG_MOD.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int reg_mod_test(int fd)
{
	int	errs	= 0;

	gsc_label("AI32SSC1M_IOCTL_REG_MOD");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


