// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/samples/sbtest/reg_read.c $
// $Rev: 53596 $
// $Date: 2023-08-23 15:57:48 -0500 (Wed, 23 Aug 2023) $

// 18AI32SSC1M: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const struct
	{
		u32	reg;
		u32	value;
	} list[]	=
	{
		// reg					value
		{ AI32SSC1M_GSC_BCTLR,	0		},
		{ AI32SSC1M_GSC_IBCR,	0		},
		{ AI32SSC1M_GSC_BCFGR,	0		},
		{ AI32SSC1M_GSC_BUFSR,	0		},
		{ AI32SSC1M_GSC_IBDR,	0		},
		{ AI32SSC1M_GSC_RAGR,	0		},
		{ AI32SSC1M_GSC_RBGR,	0		},

		{ GSC_PCI_9056_VIDR,	0x10B5	},
		{ GSC_PCI_9056_DIDR,	0x9056	},
		{ GSC_PCI_9056_SVID,	0x10B5	},

		{ GSC_PLX_9056_VIDR,	0x10B5	},
		{ GSC_PLX_9056_DIDR,	0x9056	}
	};

	int			errs	= 0;
	int			i;
	int			j;
	gsc_reg_t	parm;

	for (i = 0; i < SIZEOF_ARRAY(list); i++)
	{
		parm.reg	= list[i].reg;
		parm.value	= 0xDEADBEEF;
		parm.mask	= 0xBEEFDEAD;
		j			= ai32ssc1m_ioctl(fd, AI32SSC1M_IOCTL_REG_READ, (void*) &parm);

		if (j < 0)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, ai32ssc1m_ioctl() failure, status %d)\n",
					__LINE__,
					i,
					j);
			break;
		}

		if (parm.value == 0xDEADBEEF)
		{
			errs	= 1;
			printf("FAIL <---  (%d. i %d, value not changed)\n", __LINE__, i);
			break;
		}

		if (parm.mask != 0xBEEFDEAD)
		{
			errs	= 1;
			printf("FAIL <---  (%d. i %d, mask changed)\n", __LINE__, i);
			break;
		}

		if ((list[i].value) && (parm.value != list[i].value))
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, got 0x%lX, expected 0x%lX)\n",
					__LINE__,
					i,
					(long) parm.value,
					(long) list[i].value);
			break;
		}
	}

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	return(0);
}



/******************************************************************************
*
*	Function:	reg_read_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AI32SSC1M_IOCTL_REG_READ.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int reg_read_test(int fd)
{
	int	errs	= 0;

	gsc_label("AI32SSC1M_IOCTL_REG_READ");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


