// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/utils/util_ain_buf_clear.c $
// $Rev: 53590 $
// $Date: 2023-08-23 15:33:13 -0500 (Wed, 23 Aug 2023) $

// 18AI32SSC1M: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ai32ssc1m_ain_buf_clear
*
*	Purpose:
*
*		Provide a visual wrapper for the AI32SSC1M_IOCTL_AIN_BUF_CLEAR service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		at_bndy	Clear the buffer at a scan boundary?
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	ai32ssc1m_ain_buf_clear(int fd, int index, int verbose, int at_bndy)
{
	s32	enabled	= AI32SSC1M_CLOCK_ENABLE_NO;
	int	errs;
	int	ret;

	if (verbose)
		gsc_label_index("Input Buffer Clear", index);

	if (at_bndy)
	{
		errs	= ai32ssc1m_clock_enable(fd, -1, 0, -1, &enabled);

		// Disable data clocking as needed.

		if (enabled == AI32SSC1M_CLOCK_ENABLE_YES)
			errs	+= ai32ssc1m_clock_enable(fd, -1, 0, AI32SSC1M_CLOCK_ENABLE_NO, NULL);

		// Clear the buffer with any over or under flow status.
		ret		= ai32ssc1m_ioctl(fd, AI32SSC1M_IOCTL_AIN_BUF_CLEAR, NULL);
		errs	+= ret ? 1 : 0;

		// Enable data clocking as needed.

		if (enabled == AI32SSC1M_CLOCK_ENABLE_YES)
			errs	+= ai32ssc1m_clock_enable(fd, -1, 0, AI32SSC1M_CLOCK_ENABLE_YES, NULL);
	}
	else
	{
		ret		= ai32ssc1m_ioctl(fd, AI32SSC1M_IOCTL_AIN_BUF_CLEAR, NULL);
		errs	= ret ? 1 : 0;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}


