// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/utils/util_ain_range.c $
// $Rev: 44165 $
// $Date: 2018-11-20 14:44:06 -0600 (Tue, 20 Nov 2018) $

// 18AI32SSC1M: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ai32ssc1m_ain_range
*
*	Purpose:
*
*		Provide a visual wrapper for the AI32SSC1M_IOCTL_AIN_RANGE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai32ssc1m_ain_range(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("Analog Input Range", index);

	ret		= ai32ssc1m_ioctl(fd, AI32SSC1M_IOCTL_AIN_RANGE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AI32SSC1M_AIN_RANGE_1_25V:

			ptr	= "+-1.25v";
			break;

		case AI32SSC1M_AIN_RANGE_2_5V:

			ptr	= "+-2.5v";
			break;

		case AI32SSC1M_AIN_RANGE_5V:

			ptr	= "+-5v";
			break;

		case AI32SSC1M_AIN_RANGE_10V:

			ptr	= "+-10v";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


