// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/utils/util_autocal_bg.c $
// $Rev: 53590 $
// $Date: 2023-08-23 15:33:13 -0500 (Wed, 23 Aug 2023) $

// 18AI32SSC1M: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ai32ssc1m_autocal_bg
*
*	Purpose:
*
*		Provide a visual wrapper for the AI32SSC1M_IOCTL_AUTOCAL_BG service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai32ssc1m_autocal_bg(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("Background Calibration", index);

	ret		= ai32ssc1m_ioctl(fd, AI32SSC1M_IOCTL_AUTOCAL_BG, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AI32SSC1M_AUTOCAL_BG_DISABLE:

			ptr	= "Disabled";
			break;

		case AI32SSC1M_AUTOCAL_BG_ENABLE:

			ptr	= "Enabled";
			break;

	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


