// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/utils/util_chan_active.c $
// $Rev: 44165 $
// $Date: 2018-11-20 14:44:06 -0600 (Tue, 20 Nov 2018) $

// 18AI32SSC1M: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ai32ssc1m_chan_active
*
*	Purpose:
*
*		Provide a visual wrapper for the AI32SSC1M_IOCTL_CHAN_ACTIVE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai32ssc1m_chan_active(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("Active Channels", index);

	ret		= ai32ssc1m_ioctl(fd, AI32SSC1M_IOCTL_CHAN_ACTIVE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AI32SSC1M_CHAN_ACTIVE_SINGLE:

			ptr	= "Single Channel (software specified)";
			break;

		case AI32SSC1M_CHAN_ACTIVE_0_1:

			ptr	= "Channels 0-1";
			break;

		case AI32SSC1M_CHAN_ACTIVE_0_3:

			ptr	= "Channels 0-3";
			break;

		case AI32SSC1M_CHAN_ACTIVE_0_7:

			ptr	= "Channels 0-7";
			break;

		case AI32SSC1M_CHAN_ACTIVE_0_15:

			ptr	= "Channels 0-15";
			break;

		case AI32SSC1M_CHAN_ACTIVE_0_31:

			ptr	= "Channels 0-31";
			break;

		case AI32SSC1M_CHAN_ACTIVE_RANGE:

			ptr	= "Range of Channels (software specified)";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


