// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/utils/util_fsamp_ai.c $
// $Rev: 53590 $
// $Date: 2023-08-23 15:33:13 -0500 (Wed, 23 Aug 2023) $

// 18AI32SSC1M: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	ABS(v)					(((v) < 0) ? (-(v)) : (v))



// data types *****************************************************************

typedef struct
{
	s32			fsamp_want;
	long double	fsamp_got;

	long double	delta;
	long double	error;		// absolute of delta
	s32			enable_a;
	s32			enable_b;
	s32			master;
	s32			fsamp_max;
	s32			fsamp_min;
	s32			nrate_a;
	s32			nrate_b;
	s32			nrate_max;
	s32			nrate_min;
	s32			src;
	s32			src_b;
} compute_t;



/******************************************************************************
*
*	Function:	_channels
*
*	Purpose:
*
*		Check to see how many channels the device has.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*		first	This is the index of the first channel.
*
*		last	This is the index of the last channel.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

static int _channels(int fd, int verbose, s32* first, s32* last)
{
	s32	active;
	int	errs	= 0;
	s32	single	= -1;

	first[0]	= -1;
	last[0]		= -1;

	active	= -1;
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_CHAN_ACTIVE,	&active	);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_CHAN_FIRST,	first	);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_CHAN_LAST,	last	);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_CHAN_SINGLE,	&single	);

	if (errs == 0)
	{
		switch (active)
		{
			default:

				errs++;

				if (verbose)
					printf("FAIL <---  (invalid ACTIVE selection: %ld)\n", (long) active);

				break;

			case AI32SSC1M_CHAN_ACTIVE_0_1:

				first[0]	= 0;
				last[0]		= 1;
				break;

			case AI32SSC1M_CHAN_ACTIVE_0_3:

				first[0]	= 0;
				last[0]		= 3;
				break;

			case AI32SSC1M_CHAN_ACTIVE_0_7:

				first[0]	= 0;
				last[0]		= 7;
				break;

			case AI32SSC1M_CHAN_ACTIVE_0_15:

				first[0]	= 0;
				last[0]		= 15;
				break;

			case AI32SSC1M_CHAN_ACTIVE_0_31:

				first[0]	= 0;
				last[0]		= 31;
				break;

			case AI32SSC1M_CHAN_ACTIVE_RANGE:

				break;

			case AI32SSC1M_CHAN_ACTIVE_SINGLE:

				first[0]	= single;
				last[0]		= single;
				break;
		}
	}

	return(errs);
}



//*****************************************************************************
static int _rag_disabled(int fd, s32* sps, char* buf)
{
	strcpy(buf, "Rate-A Generator Disabled");

	if (sps)
		sps[0]	= 0;

	return(0);
}



//*****************************************************************************
static int _rbg_disabled(int fd, s32* sps, char* buf)
{
	strcpy(buf, "Rate-B Generator Disabled");

	if (sps)
		sps[0]	= 0;

	return(0);
}



//*****************************************************************************
static int _samp_bcr(int fd, s32* sps, char* buf)
{
	strcpy(buf, "Software Clocked Sampling");

	if (sps)
		sps[0]	= 0;

	return(0);
}



//*****************************************************************************
static int _samp_disabled(int fd, s32* sps, char* buf)
{
	strcpy(buf, "Clocking Disabled");

	if (sps)
		sps[0]	= 0;

	return(0);
}



//*****************************************************************************
static int _samp_ext(int fd, s32* sps, char* buf)
{
	strcpy(buf, "Externally Clocked Sampling");

	if (sps)
		sps[0]	= 0;

	return(0);
}



//*****************************************************************************
static int _samp_rag_mc(int fd, s32* sps, char* buf)
{
	s32	a_enable	= -1;
	s32	a_nrate		= -1;
	s32	clk_enable	= -1;
	int	errs		= 0;
	s32	fsamp		= 0;
	s32	fsamp_max	= -1;
	s32	master		= -1;

	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_CLOCK_ENABLE,	&clk_enable);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RAG_ENABLE,		&a_enable);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RAG_NRATE,		&a_nrate);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_MASTER_CLOCK,	&master);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_FSAMP_MAX,	&fsamp_max);

	for (;;)	// A convenience loop.
	{
		if (clk_enable == AI32SSC1M_CLOCK_ENABLE_NO)
		{
			errs	+= _samp_disabled(fd, &fsamp, buf);
			break;
		}

		if (a_enable == AI32SSC1M_GEN_ENABLE_NO)
		{
			errs	+= _rag_disabled(fd, &fsamp, buf);
			break;
		}

		sprintf(buf, "Fref %ld, A Nrate %ld", (long) master, (long) a_nrate);

		// Rate-A Generator calculations
		fsamp	= master / a_nrate;
		break;
	}

	if ((errs == 0) && (fsamp > fsamp_max))
		strcat(buf, "  <--- UNSTABLE");

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _samp_rbg_mc(int fd, s32* sps, char* buf)
{
	s32	b_enable	= -1;
	s32	b_nrate		= -1;
	s32	clk_enable	= -1;
	int	errs		= 0;
	s32	fsamp		= 0;
	s32	fsamp_max	= -1;
	s32	master		= -1;

	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_CLOCK_ENABLE,	&clk_enable);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RBG_ENABLE,		&b_enable);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RBG_NRATE,		&b_nrate);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_MASTER_CLOCK,	&master);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_FSAMP_MAX,	&fsamp_max);

	for (;;)	// A convenience loop.
	{
		if (clk_enable == AI32SSC1M_CLOCK_ENABLE_NO)
		{
			errs	+= _samp_disabled(fd, &fsamp, buf);
			break;
		}

		if (b_enable == AI32SSC1M_GEN_ENABLE_NO)
		{
			errs	+= _rbg_disabled(fd, &fsamp, buf);
			break;
		}

		sprintf(buf, "Fref %ld, B Nrate %ld", (long) master, (long) b_nrate);

		// Rate-A Generator calculations
		fsamp	= master / b_nrate;
		break;
	}

	if ((errs == 0) && (fsamp > fsamp_max))
		strcat(buf, "  <--- UNSTABLE");

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _samp_rbg_rag_mc(int fd, s32* sps, char* buf)
{
	s32	a_enable	= -1;
	s32	a_nrate		= -1;
	s32	b_enable	= -1;
	s32	b_nrate		= -1;
	s32	clk_enable	= -1;
	int	errs		= 0;
	s32	fsamp		= 0;
	s32	fsamp_max	= -1;
	s32	master		= -1;

	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_CLOCK_ENABLE,	&clk_enable);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RAG_ENABLE,		&a_enable);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RAG_NRATE,		&a_nrate);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RBG_ENABLE,		&b_enable);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RBG_NRATE,		&b_nrate);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_MASTER_CLOCK,	&master);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_FSAMP_MAX,	&fsamp_max);

	for (;;)	// A convenience loop.
	{
		if (clk_enable == AI32SSC1M_CLOCK_ENABLE_NO)
		{
			errs	+= _samp_disabled(fd, &fsamp, buf);
			break;
		}

		if (a_enable == AI32SSC1M_GEN_ENABLE_NO)
		{
			errs	+= _rag_disabled(fd, &fsamp, buf);
			break;
		}

		if (b_enable == AI32SSC1M_GEN_ENABLE_NO)
		{
			errs	+= _rbg_disabled(fd, &fsamp, buf);
			break;
		}

		sprintf(buf,
				"Fref %ld, A Nrate %ld, B Nrate %ld",
				(long) master,
				(long) a_nrate,
				(long) b_nrate);

		// Rate-A/B Generator calculations
		fsamp	= master / (a_nrate * b_nrate);
		break;
	}

	if ((errs == 0) && (fsamp > fsamp_max))
		strcat(buf, "  <--- UNSTABLE");

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _samp_rbg(int fd, s32* sps, char* buf)
{
	int	errs	= 0;
	s32	fsamp	= 0;
	s32	rbg		= -1;

	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RBG_CLK_SRC, &rbg);

	if (errs == 0)
	{
		switch (rbg)
		{
			default:

				sprintf(buf, "%d. INTERNAL ERROR: %ld", __LINE__, (long) rbg);
				break;

			case AI32SSC1M_RBG_CLK_SRC_MASTER:

				errs	+= _samp_rbg_mc(fd, &fsamp, buf);
				break;

			case AI32SSC1M_RBG_CLK_SRC_RAG:

				errs	+= _samp_rbg_rag_mc(fd, &fsamp, buf);
				break;
		}
	}

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _burst_bcr(int fd, s32* sps, char* buf)
{
	strcpy(buf, "Software Triggered Bursting");

	if (sps)
		sps[0]	= 0;

	return(0);
}



//*****************************************************************************
static int _burst_disable(int fd, s32* sps, char* buf)
{
	int	errs	= 0;
	s32	fsamp	= 0;
	s32	samp	= -1;

	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_SAMP_CLK_SRC, &samp);

	if (errs == 0)
	{
		switch (samp)
		{
			default:

				sprintf(buf, "%d. INTERNAL ERROR: %ld", __LINE__, (long) samp);
				break;

			case AI32SSC1M_SAMP_CLK_SRC_BCR:

				errs	+= _samp_bcr(fd, &fsamp, buf);
				break;

			case AI32SSC1M_SAMP_CLK_SRC_EXT:

				errs	+= _samp_ext(fd, &fsamp, buf);
				break;

			case AI32SSC1M_SAMP_CLK_SRC_RAG:

				errs	+= _samp_rag_mc(fd, &fsamp, buf);
				break;

			case AI32SSC1M_SAMP_CLK_SRC_RBG:

				errs	+= _samp_rbg(fd, &fsamp, buf);
				break;
		}
	}

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _burst_ext(int fd, s32* sps, char* buf)
{
	strcpy(buf, "Externally Triggered Bursting");

	if (sps)
		sps[0]	= 0;

	return(0);
}



//*****************************************************************************
static int _burst_rbg_mc_samp_rag_mc(int fd, s32* sps, char* buf)
{
	s32			a_enable	= -1;
	s32			a_nrate		= -1;
	s32			b_enable	= -1;
	s32			b_nrate		= -1;
	float		b_tb;				// Burst: Time Burst
	long long	b_tb_ns;			// Burst: Time Burst Nano Seconds.
	long long	b_tt_ns;			// Burst: Time Total Nano Seconds.
	float		bps;				// Bursts Per Second
	s32			clk_enable	= -1;
	int			errs		= 0;
	s32			fsamp		= 0;
	s32			fsamp_max	= -1;
	s32			master		= -1;
	long		qty;
	long long	s_tb_ns;			// Sample: Time Burst Nano Seconds.
	float		s_ts;				// Sample: Time Scan
	long long	s_ts_ns;			// Sample: Time Scan Nano Seconds.
	s32			size		= -1;

	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_CLOCK_ENABLE,	&clk_enable);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_BURST_SIZE,		&size);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RAG_ENABLE,		&a_enable);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RAG_NRATE,		&a_nrate);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RBG_ENABLE,		&b_enable);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RBG_NRATE,		&b_nrate);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_MASTER_CLOCK,	&master);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_FSAMP_MAX,	&fsamp_max);

	for (;;)	// A convenience loop.
	{
		if (clk_enable == AI32SSC1M_CLOCK_ENABLE_NO)
		{
			errs	+= _samp_disabled(fd, &fsamp, buf);
			break;
		}

		if (a_enable == AI32SSC1M_GEN_ENABLE_NO)
		{
			errs	+= _rag_disabled(fd, &fsamp, buf);
			break;
		}

		if (b_enable == AI32SSC1M_GEN_ENABLE_NO)
		{
			errs	+= _rbg_disabled(fd, &fsamp, buf);
			break;
		}

		sprintf(buf,
				"Fref %ld, A Nrate %ld, B Nrate %ld, Burst Size %ld",
				(long) master,
				(long) a_nrate,
				(long) b_nrate,
				(long) size);

		if (size == 0)
		{
			fsamp	= master / a_nrate;
			break;
		}

		// Sample clocking
		s_ts	= (float) a_nrate / master;
		s_ts_ns	= (long long) (s_ts * 1000000000L + 0.5);
		s_tb_ns	= s_ts_ns * size;

		// Burst Clocking
		b_tb	= (float) b_nrate / master;
		b_tb_ns	= (long long) (b_tb * 1000000000L + 0.5);
		qty		= (long) ((s_tb_ns + s_ts_ns - 1) / b_tb_ns);
		qty		= (qty < 1) ? 1 : qty;
		b_tt_ns	= b_tb_ns * qty;

		// Sample rate.
		bps		= 1.0 / (1.0 * b_tt_ns / 1000000000L);
		fsamp	= size * bps;
		break;
	}

	if (fsamp > fsamp_max)
		strcat(buf, "  <--- UNSTABLE");

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _burst_rbg_mc_samp_rbg_mc(int fd, s32* sps, char* buf)
{
	s32			b_enable	= -1;
	s32			b_nrate		= -1;
	float		b_tb;				// Burst: Time Burst
	long long	b_tb_ns;			// Burst: Time Burst Nano Seconds.
	long long	b_tt_ns;			// Burst: Time Total Nano Seconds.
	float		bps;				// Bursts Per Second
	s32			clk_enable	= -1;
	int			errs		= 0;
	s32			fsamp		= 0;
	s32			fsamp_max	= -1;
	s32			master		= -1;
	long		qty;
	long long	s_tb_ns;			// Sample: Time Burst Nano Seconds.
	float		s_ts;				// Sample: Time Scan
	long long	s_ts_ns;			// Sample: Time Scan Nano Seconds.
	s32			size		= -1;

	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_CLOCK_ENABLE,	&clk_enable);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_BURST_SIZE,		&size);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RBG_ENABLE,		&b_enable);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RBG_NRATE,		&b_nrate);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_MASTER_CLOCK,	&master);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_FSAMP_MAX,	&fsamp_max);

	for (;;)	// A convenience loop.
	{
		if (clk_enable == AI32SSC1M_CLOCK_ENABLE_NO)
		{
			errs	+= _samp_disabled(fd, &fsamp, buf);
			break;
		}

		if (b_enable == AI32SSC1M_GEN_ENABLE_NO)
		{
			errs	+= _rbg_disabled(fd, &fsamp, buf);
			break;
		}

		sprintf(buf,
				"Fref %ld, B Nrate %ld, Burst Size %ld",
				(long) master,
				(long) b_nrate,
				(long) size);

		if (size == 0)
		{
			fsamp	= master / b_nrate;
			break;
		}

		// Sample clocking
		s_ts	= (float) b_nrate / master;
		s_ts_ns	= (long long) (s_ts * 1000000000L + 0.5);
		s_tb_ns	= s_ts_ns * size;

		// Burst Clocking
		b_tb	= (float) b_nrate / master;
		b_tb_ns	= (long long) (b_tb * 1000000000L + 0.5);
		qty		= (long) ((s_tb_ns + s_ts_ns - 1) / b_tb_ns);
		qty		= (qty < 1) ? 1 : qty;
		b_tt_ns	= b_tb_ns * qty;

		// Sample rate.
		bps		= 1.0 / (1.0 * b_tt_ns / 1000000000L);
		fsamp	= size * bps;
		break;
	}

	if (fsamp > fsamp_max)
		strcat(buf, "  <--- UNSTABLE");

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _burst_rbg_mc(int fd, s32* sps, char* buf)
{
	s32	adc		= -1;
	int	errs	= 0;
	s32	fsamp	= 0;

	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_SAMP_CLK_SRC, &adc);

	if (errs == 0)
	{
		switch (adc)
		{
			default:

				sprintf(buf, "%d. INTERNAL ERROR: %ld", __LINE__, (long) adc);
				break;

			case AI32SSC1M_SAMP_CLK_SRC_BCR:

				errs	+= _samp_bcr(fd, &fsamp, buf);
				break;

			case AI32SSC1M_SAMP_CLK_SRC_EXT:

				errs	+= _samp_ext(fd, &fsamp, buf);
				break;

			case AI32SSC1M_SAMP_CLK_SRC_RAG:

				errs	+= _burst_rbg_mc_samp_rag_mc(fd, &fsamp, buf);
				break;

			case AI32SSC1M_SAMP_CLK_SRC_RBG:

				errs	+= _burst_rbg_mc_samp_rbg_mc(fd, &fsamp, buf);
				break;
		}
	}

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _burst_rbg_rag_mc_samp_rag_mc(int fd, s32* sps, char* buf)
{
	s32			a_enable	= -1;
	s32			a_nrate		= -1;
	s32			b_enable	= -1;
	s32			b_nrate		= -1;
	float		b_tb;				// Burst: Time Burst
	long long	b_tb_ns;			// Burst: Time Burst Nano Seconds.
	long long	b_tt_ns;			// Burst: Time Total Nano Seconds.
	float		bps;				// Bursts Per Second
	s32			clk_enable	= -1;
	int			errs		= 0;
	s32			fsamp		= 0;
	s32			fsamp_max	= -1;
	s32			master		= -1;
	long		qty;
	long long	s_tb_ns;			// Sample: Time Burst Nano Seconds.
	float		s_ts;				// Sample: Time Scan
	long long	s_ts_ns;			// Sample: Time Scan Nano Seconds.
	s32			size		= -1;

	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_CLOCK_ENABLE,	&clk_enable);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_BURST_SIZE,		&size);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RAG_ENABLE,		&a_enable);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RAG_NRATE,		&a_nrate);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RBG_ENABLE,		&b_enable);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RBG_NRATE,		&b_nrate);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_MASTER_CLOCK,	&master);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_FSAMP_MAX,	&fsamp_max);

	for (;;)	// A convenience loop.
	{
		if (clk_enable == AI32SSC1M_CLOCK_ENABLE_NO)
		{
			errs	+= _samp_disabled(fd, &fsamp, buf);
			break;
		}

		if (a_enable == AI32SSC1M_GEN_ENABLE_NO)
		{
			errs	+= _rag_disabled(fd, &fsamp, buf);
			break;
		}

		if (b_enable == AI32SSC1M_GEN_ENABLE_NO)
		{
			errs	+= _rbg_disabled(fd, &fsamp, buf);
			break;
		}

		sprintf(buf,
				"Fref %ld, A Nrate %ld, B Nrate %ld, Burst Size %ld",
				(long) master,
				(long) a_nrate,
				(long) b_nrate,
				(long) size);

		if (size == 0)
		{
			fsamp	= master / a_nrate;
			break;
		}

		// Sample clocking
		s_ts	= (float) a_nrate / master;
		s_ts_ns	= (long long) (s_ts * 1000000000L + 0.5);
		s_tb_ns	= s_ts_ns * size;

		// Burst Clocking
		b_tb	= (float) (a_nrate * b_nrate) / master;
		b_tb_ns	= (long long) (b_tb * 1000000000L + 0.5);
		qty		= (long) ((s_tb_ns + s_ts_ns - 1) / b_tb_ns);
		qty		= (qty < 1) ? 1 : qty;
		b_tt_ns	= b_tb_ns * qty;

		// Sample rate.
		bps		= 1.0 / (1.0 * b_tt_ns / 1000000000L);
		fsamp	= size * bps;
		break;
	}

	if (fsamp > fsamp_max)
		strcat(buf, "  <--- UNSTABLE");

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _burst_rbg_rag_mc_samp_rbg_rag_mc(int fd, s32* sps, char* buf)
{
	s32			a_enable	= -1;
	s32			a_nrate		= -1;
	s32			b_enable	= -1;
	s32			b_nrate		= -1;
	float		b_tb;				// Burst: Time Burst
	long long	b_tb_ns;			// Burst: Time Burst Nano Seconds.
	long long	b_tt_ns;			// Burst: Time Total Nano Seconds.
	float		bps;				// Bursts Per Second
	s32			clk_enable	= -1;
	int			errs		= 0;
	s32			fsamp		= 0;
	s32			fsamp_max	= -1;
	s32			master		= -1;
	long		qty;
	long long	s_tb_ns;			// Sample: Time Burst Nano Seconds.
	float		s_ts;				// Sample: Time Scan
	long long	s_ts_ns;			// Sample: Time Scan Nano Seconds.
	s32			size		= -1;

	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_CLOCK_ENABLE,	&clk_enable);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_BURST_SIZE,		&size);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RAG_ENABLE,		&a_enable);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RAG_NRATE,		&a_nrate);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RBG_ENABLE,		&b_enable);
	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RBG_NRATE,		&b_nrate);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_MASTER_CLOCK,	&master);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_FSAMP_MAX,	&fsamp_max);

	for (;;)	// A convenience loop.
	{
		if (clk_enable == AI32SSC1M_CLOCK_ENABLE_NO)
		{
			errs	+= _samp_disabled(fd, &fsamp, buf);
			break;
		}

		if (a_enable == AI32SSC1M_GEN_ENABLE_NO)
		{
			errs	+= _rag_disabled(fd, &fsamp, buf);
			break;
		}

		if (b_enable == AI32SSC1M_GEN_ENABLE_NO)
		{
			errs	+= _rbg_disabled(fd, &fsamp, buf);
			break;
		}

		sprintf(buf,
				"Fref %ld, A Nrate %ld, B Nrate %ld, Burst Size %ld",
				(long) master,
				(long) a_nrate,
				(long) b_nrate,
				(long) size);

		if (size == 0)
		{
			fsamp	= master / a_nrate;
			break;
		}

		// Sample clocking
		s_ts	= (float) (a_nrate * b_nrate) / master;
		s_ts_ns	= (long long) (s_ts * 1000000000L + 0.5);
		s_tb_ns	= s_ts_ns * size;

		// Burst Clocking
		b_tb	= (float) (a_nrate * b_nrate) / master;
		b_tb_ns	= (long long) (b_tb * 1000000000L + 0.5);
		qty		= (long) ((s_tb_ns + s_ts_ns - 1) / b_tb_ns);
		qty		= (qty < 1) ? 1 : qty;
		b_tt_ns	= b_tb_ns * qty;

		// Sample rate.
		bps		= 1.0 / (1.0 * b_tt_ns / 1000000000L);
		fsamp	= size * bps;
		break;
	}

	if (fsamp > fsamp_max)
		strcat(buf, "  <--- UNSTABLE");

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _burst_rbg_rag_mc(int fd, s32* sps, char* buf)
{
	s32	adc		= -1;
	int	errs	= 0;
	s32	fsamp	= 0;

	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_SAMP_CLK_SRC, &adc);

	if (errs == 0)
	{
		switch (adc)
		{
			default:

				sprintf(buf, "%d. INTERNAL ERROR: %ld", __LINE__, (long) adc);
				break;

			case AI32SSC1M_SAMP_CLK_SRC_BCR:

				errs	+= _samp_bcr(fd, &fsamp, buf);
				break;

			case AI32SSC1M_SAMP_CLK_SRC_EXT:

				errs	+= _samp_ext(fd, &fsamp, buf);
				break;

			case AI32SSC1M_SAMP_CLK_SRC_RAG:

				errs	+= _burst_rbg_rag_mc_samp_rag_mc(fd, &fsamp, buf);
				break;

			case AI32SSC1M_SAMP_CLK_SRC_RBG:

				errs	+= _burst_rbg_rag_mc_samp_rbg_rag_mc(fd, &fsamp, buf);
				break;
		}
	}

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _burst_rbg(int fd, s32* sps, char* buf)
{
	int	errs	= 0;
	s32	fsamp	= 0;
	s32	rbg		= -1;

	errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_RBG_CLK_SRC, &rbg);

	if (errs == 0)
	{
		switch (rbg)
		{
			default:

				sprintf(buf, "%d. INTERNAL ERROR: %ld", __LINE__, (long) rbg);
				break;

			case AI32SSC1M_RBG_CLK_SRC_MASTER:

				errs	+= _burst_rbg_mc(fd, &fsamp, buf);
				break;

			case AI32SSC1M_RBG_CLK_SRC_RAG:

				errs	+= _burst_rbg_rag_mc(fd, &fsamp, buf);
				break;
		}
	}

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static void _fsamp_compute_a(compute_t* data)
{
	long double	delta;
	long double	error;
	long double	fsamp_got;
	long double	fsamp_want	= data->fsamp_want;
	int			init		= 0;	// Have we performed the first calculation?
	s32			master		= data->master;
	s32			nr;
	s32			nr_max;
	s32			nr_min;
	s32			nrate_a;
	s32			nrate_max	= data->nrate_max;
	s32			nrate_min	= data->nrate_min;

	data->src		= AI32SSC1M_SAMP_CLK_SRC_RAG;
	data->src_b		= AI32SSC1M_RBG_CLK_SRC_MASTER;
	data->enable_a	= AI32SSC1M_GEN_ENABLE_YES;
	data->enable_b	= AI32SSC1M_GEN_ENABLE_NO;

	nr		= master / fsamp_want;
	nr		= (nr < nrate_min) ? nrate_min
			: (nr > nrate_max) ? nrate_max : nr;

	nr_min	= nr - 5;
	nr_max	= nr + 5;
	nr_min	= (nr_min < nrate_min) ? nrate_min
			: (nr_min > nrate_max) ? nrate_max : nr_min;
	nr_max	= (nr_max < nrate_min) ? nrate_min
			: (nr_max > nrate_max) ? nrate_max : nr_max;

	for (nrate_a = nr_min; nrate_a <= nr_max; nrate_a++)
	{
		fsamp_got	= (long double) master / nrate_a;
		delta		= fsamp_got - fsamp_want;
		error		= ABS(delta);

		if ((init == 0) || (error < data->error))
		{
			init			= 1;
			data->delta		= delta;
			data->error		= error;
			data->fsamp_got	= fsamp_got;
			data->nrate_a	= nrate_a;
		}
	}
}



//*****************************************************************************
static void _fsamp_compute_ab(compute_t* data)
{
	long double	delta;
	long double	error;
	long double	fsamp_got;
	long double	fsamp_want	= data->fsamp_want;
	int			init		= 0;	// Have we performed the first calculation?
	s32			master		= data->master;
	s32			nr;
	s32			nr_max;
	s32			nr_min;
	s32			nrate_a;
	s32			nrate_b;
	s32			nrate_max	= data->nrate_max;
	s32			nrate_min	= data->nrate_min;

	data->src		= AI32SSC1M_SAMP_CLK_SRC_RBG;
	data->src_b		= AI32SSC1M_RBG_CLK_SRC_RAG;
	data->enable_a	= AI32SSC1M_GEN_ENABLE_YES;
	data->enable_b	= AI32SSC1M_GEN_ENABLE_YES;

	for (nrate_a = nrate_min; nrate_a <= nrate_max; nrate_a++)
	{
		nr		= master / fsamp_want / nrate_a;
		nr		= (nr < nrate_min) ? nrate_min
				: (nr > nrate_max) ? nrate_max : nr;

		nr_min	= nr - 5;
		nr_max	= nr + 5;
		nr_min	= (nr_min < nrate_min) ? nrate_min
				: (nr_min > nrate_max) ? nrate_max : nr_min;
		nr_max	= (nr_max < nrate_min) ? nrate_min
				: (nr_max > nrate_max) ? nrate_max : nr_max;

		for (nrate_b = nr_min; nrate_b <= nr_max; nrate_b++)
		{
			fsamp_got	= (long double) master / ((long long) nrate_b * nrate_a);
			delta		= fsamp_got - fsamp_want;
			error		= ABS(delta);

			if ((init == 0) || (error < data->error))
			{
				init			= 1;
				data->delta		= delta;
				data->error		= error;
				data->fsamp_got	= fsamp_got;
				data->nrate_a	= nrate_a;
				data->nrate_b	= nrate_b;
			}
		}
	}
}



//*****************************************************************************
int	ai32ssc1m_fsamp_ai_compute(
	int		fd,
	int		index,
	int		verbose,
	s32		fsamp,
	s32*	src,
	s32*	src_b,
	s32*	nrate_a,
	s32*	nrate_b,
	s32*	enable_a,
	s32*	enable_b,
	double*	rate)
{
	compute_t	data;
	compute_t	data_a;
	compute_t	data_ab;
	int			errs		= 0;

	if (verbose)
		gsc_label_index("Fsamp AI Compute", index);

	memset(&data, 0, sizeof(data));
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_FSAMP_MAX, &data.fsamp_max);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_FSAMP_MIN, &data.fsamp_min);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_MASTER_CLOCK, &data.master);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_NRATE_MAX, &data.nrate_max);
	errs	+= ai32ssc1m_query(fd, -1, 0, AI32SSC1M_QUERY_NRATE_MIN, &data.nrate_min);

	data.error		= LDBL_MAX;
	data.fsamp_want	= fsamp;

	data_a	= data;
	data_ab	= data;

	_fsamp_compute_a(&data_a);
	_fsamp_compute_ab(&data_ab);

	if (data_a.error < data_ab.error)
	{
		data_a.nrate_b	= data.nrate_max;
		data			= data_a;

		if (verbose)
		{
			printf("%s  (", errs ? "FAIL <---" : "PASS");
			gsc_label_long_comma(fsamp);
			printf(" S/S: ");
			printf("Fref ");
			gsc_label_long_comma((long) data.master);
			printf(", Nrate-A ");
			gsc_label_long_comma((long) data.nrate_a);
			printf(", Fsamp %.3Lf", data.fsamp_got);
			printf(")\n");
		}
	}
	else
	{
		data	= data_ab;

		if (verbose)
		{
			printf("%s  (", errs ? "FAIL <---" : "PASS");
			gsc_label_long_comma(fsamp);
			printf(" S/S: ");
			printf("Fref ");
			gsc_label_long_comma((long) data.master);
			printf(", Nrate-A ");
			gsc_label_long_comma((long) data.nrate_a);
			printf(", Nrate-B ");
			gsc_label_long_comma((long) data.nrate_b);
			printf(", Fsamp %.3Lf", data.fsamp_got);
			printf(")\n");
		}
	}

	src[0]		= data.src;
	src_b[0]	= data.src_b;
	nrate_a[0]	= data.nrate_a;
	nrate_b[0]	= data.nrate_b;
	enable_a[0]	= data.enable_a;
	enable_b[0]	= data.enable_b;

	if (rate)
		rate[0]	= (double) data.fsamp_got;

	return(errs);
}



/******************************************************************************
*
*	Function:	ai32ssc1m_fsamp_report
*
*	Purpose:
*
*		Determine and report the sample rate for the specified channel.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		chan	The index of the channel of interest.
*
*		sps		Store the sample rate here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int ai32ssc1m_fsamp_report(int fd, int index, int verbose, int chan, s32* sps)
{
	char	buf[1024];
	s32		burst		= -1;
	int		errs		= 0;
	s32		first;
	s32		fsamp		= 0;
	s32		last;

	if (verbose)
	{
		sprintf(buf, "Channel %d", chan);
		gsc_label_index(buf, index);
	}

	for (;;)
	{
		if ((chan < 0) || (chan > 32))
		{
			errs++;
			printf("FAIL <---  (invalid channel index: %d)\n", chan);
			break;
		}

		errs	+= _channels(fd, verbose, &first, &last);

		if (errs)
			break;

		if ((chan < first) || (chan > last))
		{
			sps[0]	= 0;

			if (verbose)
				printf("SKIPPED  (disabled)\n");

			break;
		}

		errs	+= ai32ssc1m_ioctl_util(fd, -1, 0, AI32SSC1M_IOCTL_BURST_SYNC, &burst);

		if (errs)
			break;

		switch (burst)
		{
			default:

				sprintf(buf, "%d. INTERNAL ERROR: %ld", __LINE__, (long) burst);
				break;

			case AI32SSC1M_BURST_SYNC_BCR:

				errs	+= _burst_bcr(fd, &fsamp, buf);
				break;

			case AI32SSC1M_BURST_SYNC_DISABLE:

				errs	+= _burst_disable(fd, &fsamp, buf);
				break;

			case AI32SSC1M_BURST_SYNC_EXT:

				errs	+= _burst_ext(fd, &fsamp, buf);
				break;

			case AI32SSC1M_BURST_SYNC_RBG:

				errs	+= _burst_rbg(fd, &fsamp, buf);
				break;
		}

		if ((errs == 0) && (verbose))
		{
			gsc_label_long_comma((long) fsamp);
			printf(" S/S  (%s)\n", buf);
		}

		break;
	}

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



/******************************************************************************
*
*	Function:	ai32ssc1m_fsamp_report_all
*
*	Purpose:
*
*		Determine and report the sample rate for all channels.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	Report the overall sample rate here.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int ai32ssc1m_fsamp_report_all(int fd, int index, int verbose, s32* fsamp)
{
	int	chan;
	int	errs	= 0;
	s32	first;
	s32	last;
	s32	sps;
	s32	total	= 0;

	if (verbose)
		gsc_label_index("Sample Rates", index);

	errs	+= _channels(fd, verbose, &first, &last);

	if (errs == 0)
	{
		if (verbose)
		{
			printf("\n");
			gsc_label_level_inc();
		}

		for (chan = first; chan <= last; chan++)
		{
			errs	+= ai32ssc1m_fsamp_report(fd, -1, 1, chan, &sps);
			total	+= sps;
		}

		if (verbose)
		{
			gsc_label("Overall Rate");
			gsc_label_long_comma(total);
			printf(" S/S\n");
			gsc_label_level_dec();
		}
	}

	if (fsamp)
		fsamp[0]	= total;

	return(errs);
}


