// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/utils/util_irq0_sel.c $
// $Rev: 53590 $
// $Date: 2023-08-23 15:33:13 -0500 (Wed, 23 Aug 2023) $

// 18AI32SSC1M: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ai32ssc1m_irq0_sel
*
*	Purpose:
*
*		Provide a visual wrapper for the AI32SSC1M_IOCTL_IRQ0_SEL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai32ssc1m_irq0_sel(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("IRQ 0 Select", index);

	ret		= ai32ssc1m_ioctl(fd, AI32SSC1M_IOCTL_IRQ0_SEL, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AI32SSC1M_IRQ0_INIT_DONE:

			ptr	= "Initialization Done";
			break;

		case AI32SSC1M_IRQ0_AUTOCAL_DONE:

			ptr	= "Autocalibration Done";
			break;

		case AI32SSC1M_IRQ0_SYNC_START:

			ptr	= "Sync Start";
			break;

		case AI32SSC1M_IRQ0_SYNC_DONE:

			ptr	= "Sync Done";
			break;

		case AI32SSC1M_IRQ0_BURST_START:

			ptr	= "Burst Start";
			break;

		case AI32SSC1M_IRQ0_BURST_DONE:

			ptr	= "Burst Done";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


