// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/utils/util_reg.c $
// $Rev: 53590 $
// $Date: 2023-08-23 15:33:13 -0500 (Wed, 23 Aug 2023) $

// 18AI32SSC1M: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	_GSC_REG(a)							"GSC " #a, AI32SSC1M_GSC_##a, 0, 0
#define	SIZEOF_ARRAY(a)						(sizeof((a)) / sizeof((a)[0]))
#define	FIELD_WIDTH							20
#define	NAME_WIDTH							(width + 10)



// prototypes *****************************************************************

static const gsc_reg_def_t*	_find_reg(u32 reg, const gsc_reg_def_t* list);

static	int	_acar_decode	(int fd, int supported, u32 value, int width);
static	int	_asiocr_decode	(int fd, int supported, u32 value, int width);
static	int	_bctlr_decode	(int fd, int supported, u32 value, int width);
static	int	_bcfgr_decode	(int fd, int supported, u32 value, int width);
static	int	_brstsr_decode	(int fd, int supported, u32 value, int width);
static	int	_bufsr_decode	(int fd, int supported, u32 value, int width);
static	int	_ecdr_decode	(int fd, int supported, u32 value, int width);
static	int	_ibcr_decode	(int fd, int supported, u32 value, int width);
static	int	_ibdr_decode	(int fd, int supported, u32 value, int width);
static	int	_icr_decode		(int fd, int supported, u32 value, int width);
static	int	_ptchr_decode	(int fd, int supported, u32 value, int width);
static	int	_ptclr_decode	(int fd, int supported, u32 value, int width);
static	int	_rgr_decode		(int fd, int supported, u32 value, int width);
static	int	_smwr_decode	(int fd, int supported, u32 value, int width);
static	int	_sscr_decode	(int fd, int supported, u32 value, int width);



// variables ******************************************************************

static gsc_reg_def_t	_gsc[]	=
{
	{ _GSC_REG(BCTLR),	0,	_bctlr_decode,	"Board Control Register"				},
	{ _GSC_REG(ICR),	0,	_icr_decode,	"Interrupt Control Register"			},
	{ _GSC_REG(IBDR),	0,	_ibdr_decode,	"Input Buffer Data Register"			},
	{ _GSC_REG(IBCR),	0,	_ibcr_decode,	"Input Buffer Control Register"			},
	{ _GSC_REG(RAGR),	0,	_rgr_decode,	"Rate-A Generator Register"				},
	{ _GSC_REG(RBGR),	0,	_rgr_decode,	"Rate-B Generator Register"				},
	{ _GSC_REG(BUFSR),	0,	_bufsr_decode,	"Buffer Size Register"					},
	{ _GSC_REG(BSTSR),	0,	_brstsr_decode,	"Burst Size Register"					},
	{ _GSC_REG(SSCR),	0,	_sscr_decode,	"Scan & Sync Control Register"			},
	{ _GSC_REG(ACAR),	0,	_acar_decode,	"Active Channel Assignment Register"	},
	{ _GSC_REG(BCFGR),	0,	_bcfgr_decode,	"Board Configuration Register"			},
	{ _GSC_REG(AVR),	0,	NULL,			"Autocal Values Register"				},
	{ _GSC_REG(ARWR),	0,	NULL,			"Auxiliary Read/Write Register"			},
	{ _GSC_REG(ASIOCR),	0,	_asiocr_decode,	"Auxiliary Sync I/O Control Register"	},
	{ _GSC_REG(SMUWR),	0,	_smwr_decode,	"Scan Marker Upper Word Register"		},
	{ _GSC_REG(SMLWR),	0,	_smwr_decode,	"Scan Marker Lower Word Register"		},
	{ _GSC_REG(TUR),	0,	NULL,			"Test Utility Register"					},
	{ _GSC_REG(PTCLR),	1,	_ptclr_decode,	"Pretrigger Counter Low Register"		},
	{ _GSC_REG(PTCHR),	1,	_ptchr_decode,	"Pretrigger Counter High Register"		},
	{ _GSC_REG(ECDR),	1,	_ecdr_decode,	"External Clock Divisor Register"		},

	{ NULL, 0, 0, 0, 0,		NULL	}
};



//*****************************************************************************
static int _bctlr_decode(int fd, int supported, u32 value, int width)
{
	static const char * const	aim[]	=
	{
		"Differential",
		"RESERVED",
		"Zero Test",
		"+Vref Test",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char * const	range[]	=
	{
		"RESERVED",
		"Low (+-5 Volts/+-1.25 Volts)",
		"Hi (+-10 Volts/+-2.5 Volts)",
		"Hi (+-10 Volts/+-2.5 Volts)"
	};

	static const char * const	disabled[]	= { "Disabled",			"Enabled"		};
	static const char * const	enabled[]	= { "Enabled",			"Disabled"		};
	static const char * const	idle[]		= { "Idle",				"Active"		};
	static const char * const	no[]		= { "No",				"Yes"			};
	static const char * const	twos[]		= { "Twos Compliment",	"Offset Binary"	};

	const gsc_reg_def_t*	bcfgr;
	int						errs	= 0;

	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 31, 19, 1, NULL,		"Reserved"			);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 18, 18, 1, disabled,	"Data Packing"		);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 17, 17, 1, no,		"Buffer Overflow"	);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 16, 16, 1, no,		"Buffer Underflow"	);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 15, 15, 1, idle,		"Initialize"		);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 14, 14, 1, no,		"Autocal Pass"		);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 13, 13, 1, idle,		"Autocal"			);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 12, 12, 1, idle,		"Input Sync"		);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 11, 11, 1, enabled,	"Scan Marker"		);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 10,  9, 1, NULL,		"Reserved"			);

	bcfgr	= _find_reg(AI32SSC1M_GSC_BCFGR, _gsc);

	if ((bcfgr == NULL) || ((bcfgr->value & 0xFFF) < 0x002))
	{
		errs++;
		gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  8,  8, 1, NULL,	"Reserved"	);
	}
	else
	{
		gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  8,  8, 1, disabled,	"Background Cal"	);
	}

	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  7,  7, 1, NULL,		"Reserved"			);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  6,  6, 1, twos,		"Data Format"		);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  5,  4, 1, range,	"Range"				);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  3,  3, 1, NULL,		"Reserved"			);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  2,  0, 1, aim,		"Input Mode"		);

	return(errs);
}



//*****************************************************************************
static int _icr_decode(int fd, int supported, u32 value, int width)
{
	static const char * const	irq0[]	=
	{
		"Initialization Complete",
		"Autocal Complete",
		"Input Sample Initiated (Sync)",
		"Input Sample Completed (data ready)",
		"Trigger Burst Initiated (Burst Busy -> High)",
		"Trigger Burst Completed (Burst Busy -> Low)",
		"Background Calibrate Fsamp exceeded",
		"Reserved"
	};

	static const char * const	irq1[]	=
	{
		"Idle/None",
		"Input Buffer Threshold Low->High",
		"Input Buffer Threshold High->Low",
		"Input Buffer Overflow/Underflow",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved"
	};

	static const char * const	negated[]	= { "Negated",	"Asserted"	};

	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 31,  8, 1, NULL,		"Reserved"		);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  7,  7, 1, negated,	"IRQ1 Request"	);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  6,  4, 1, irq1,		"IRQ1 Select"	);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  3,  3, 1, negated,	"IRQ0 Request"	);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  2,  0, 1, irq0,		"IRQ0 Select"	);

	return(0);
}



//*****************************************************************************
static int _ibdr_decode(int fd, int supported, u32 value, int width)
{
	static const char * const	tag[]	= { "Not First",	"First"	};

	const gsc_reg_def_t*	bcfgr;
	int						errs	= 0;
	const gsc_reg_def_t*	ibcr;

	bcfgr	= _find_reg(AI32SSC1M_GSC_BCFGR, _gsc);
	ibcr	= _find_reg(AI32SSC1M_GSC_IBCR, _gsc);

	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 31, 31, 1, tag,	"Channel Tag"	);

	if ((bcfgr == NULL) || (ibcr == NULL))
	{
		errs++;
		gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 30, 18, 1, NULL,	"Reserved"	);
		gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 17, 17, 1, NULL,	"ERROR"		);
		gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 16, 16, 1, NULL,	"ERROR"		);
		gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 15,  0, 1, NULL,	"Data"		);
	}
	else if ((bcfgr->value & 0x100000) && (ibcr->value & 0x100000))
	{
		// This is an 18-bit device in 18-bit mode.
		gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 30, 18, 1, NULL,	"Reserved"	);
		gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 17,  0, 1, NULL,	"Data"		);
	}
	else
	{
		// The device is in 16-bit mode.
		gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 30, 16, 1, NULL,	"Reserved"	);
		gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 15,  0, 1, NULL,	"Data"		);
	}

	return(errs);
}



//*****************************************************************************
static int _ibcr_decode(int fd, int supported, u32 value, int width)
{
	static const char * const	_16[]	= { "16-bit",	"18-bit"	};
	static const char * const	clear[]	= { "Clear",	"Set"		};
	static const char * const	idle[]	= { "Idle",		"Active"	};

	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 31, 21, 1, NULL,		"Reserved"		);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 20, 20, 1, _16,		"Data Size"		);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 19, 19, 1, clear,	"Threshold Flag");
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 18, 18, 1, idle,		"Clear Buffer"	);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 17,  0, 1, NULL,		"Threshold"		);

	return(0);
}



//*****************************************************************************
static int _rgr_decode(int fd, int supported, u32 value, int width)
{
	static const char * const	enabled[]	= { "Enabled",	"Disabled"	};

	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 31, 17, 1, NULL,		"Reserved"	);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 16, 16, 1, enabled,	"Disable"	);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 15,  0, 1, NULL,		"Nrate"		);

	return(0);
}



//*****************************************************************************
static int _bufsr_decode(int fd, int supported, u32 value, int width)
{
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 31, 19, 1, NULL,	"Reserved"			);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 18,  0, 1, NULL,	"Filled Locations"	);

	return(0);
}



//*****************************************************************************
static int _brstsr_decode(int fd, int supported, u32 value, int width)
{
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 31, 19, 1, NULL,	"Reserved"		);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 19,  0, 1, NULL,	"Burst Size"	);

	return(0);
}



//*****************************************************************************
static int _sscr_decode(int fd, int supported, u32 value, int width)
{
	static const char * const	burst[]	=
	{
		"Disabled",
		"Rate-B Generator",
		"Ext Sync Input (target Mode)",
		"Software (BCTLR Input Sync Bit)"
	};

	static const char * const	clk_src[]	=
	{
		"External Clock (clock Target mode)",
		"Rate-A Generator",
		"Rate-B Generator",
		"Software (BCTLR Input Sync Bit)"
	};

	static const char * const	channels[]	=
	{
		"Single Channel Mode",
		"2 Channels (0-1)",
		"4 Channels (0-3)",
		"8 Channels (0-7)",
		"16 Channels (0-15)",
		"32 Channels (0-31)",
		"Reserved",
		"Channel Range (x-y)"
	};

	static const char * const	b_src[]		= { "Master Clock",	"Rate-A Generator"	};
	static const char * const	disabled[]	= { "Disabled",		"Enabled"			};
	static const char * const	idle[]		= { "Idle",			"Active"			};
	static const char * const	no[]		= { "No",			"Yes"				};

	const gsc_reg_def_t*	bcfgr;
	int						errs	= 0;

	bcfgr	= _find_reg(AI32SSC1M_GSC_BCFGR, _gsc);

	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 31, 20, 1, NULL,		"Reserved"	);

	if ((bcfgr == NULL) || ((bcfgr->value & 0xFFF) < 0x004))
	{
		errs++;
		gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 19, 19, 1, NULL,	"Reserved"	);
		gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 18, 18, 1, NULL,	"Reserved"	);
	}
	else
	{
		gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 19, 19, 1, idle,		"Pretrig Latch"	);
		gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 18, 18, 1, disabled,	"Pretrig Enable");
	}

	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 17, 12, 1, NULL,		"Single Channel"		);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 11, 11, 1, no,		"Invert I/O"			);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 10, 10, 1, b_src,	"Rate-B Gen. Source"	);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  9,  8, 1, burst,	"Bursting Source"		);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  7,  7, 1, idle,		"Bursting"				);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  6,  6, 1, NULL,		"Reserved"				);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  5,  5, 1, disabled,	"Sample Clocking"		);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  4,  3, 1, clk_src,	"Sample Clock Source"	);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  2,  0, 1, channels,	"Active Channels"		);

	return(errs);
}



//*****************************************************************************
static int _acar_decode(int fd, int supported, u32 value, int width)
{
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 31, 16, 1, NULL,	"Reserved"		);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 15,  8, 1, NULL,	"Last Channel"	);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  7,  0, 1, NULL,	"First Channel"	);

	return(0);
}



//*****************************************************************************
static int _bcfgr_decode(int fd, int supported, u32 value, int width)
{
	static const char * const	channels[]	=
	{
		"32 Channels",
		"16 Channels",
		"8 Channels",
		"4 Channels"
	};

	static const char * const	master[]	=
	{
		"36 MHz",
		"Reserved",
		"Reserved",
		"Reserved"
	};

	static const char * const	_16[]	= { "16-bit",	"18-bit"		};
	static const char * const	id[]	= { "ERROR",	"18AI32SSC1M"	};
	static const char * const	range[]	= { "10/5V",	"2.5/1.25V"		};

	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 31, 21, 1, NULL,		"Reserved"			);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 20, 20, 1, _16,		"Data Size"			);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 19, 19, 1, range,	"Voltage Range"		);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 18, 18, 1, master,	"Master Clock"		);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 17, 16, 1, channels,	"Channels"			);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 15, 15, 1, id,		"Product Id"		);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 14, 12, 1, NULL,		"Reserved"			);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 11,  0, 1, NULL,		"Firmware Rev"		);

	return(0);
}



//*****************************************************************************
static int _asiocr_decode(int fd, int supported, u32 value, int width)
{
	static const char * const	clock[]	=
	{
		"Inactive",
		"Active Input (Low-to-High Edge)",
		"Active Output (Positive Pulse)",
		"Reserved"
	};

	static const char * const	noise[]	= { "100ns-135ns",		"1.5us-2.0us"		};
	static const char * const	high[]	= { "High Pulses",		"Low Pulses"		};
	static const char * const	low[]	= { "Low-to-High Edge",	"High-to-Low Edge"	};

	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 31, 11, 1, NULL,		"Reserved"			);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 10, 10, 1, noise,	"Noise Supression"	);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  9,  9, 1, high,		"Output Polarity"	);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  8,  8, 1, low,		"Input Polarity"	);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  7,  4, 1, NULL,		"Reserved"			);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  3,  2, 1, clock,	"Aux Sync Mode"		);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value,  1,  0, 1, clock,	"Aux Clock Mode"	);

	return(0);
}



//*****************************************************************************
static int _smwr_decode(int fd, int supported, u32 value, int width)
{
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 31, 16, 1, NULL,	"Reserved"	);
	gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 15,  0, 1, NULL,	"Marker"	);

	return(0);
}



//*****************************************************************************
static int _ptclr_decode(int fd, int supported, u32 value, int width)
{
	const gsc_reg_def_t*	bcfgr;
	int						ret;

	if (supported)
	{
		bcfgr	= _find_reg(AI32SSC1M_GSC_BCFGR, _gsc);
		ret		= bcfgr
				? (((bcfgr->value & 0xFFF) > 4) ? 1 : 0)
				: 0;
	}
	else
	{
		ret	= 0;
		gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 31, 0, 1, NULL,	"Low Count"	);
	}

	return(ret);
}



//*****************************************************************************
static int _ptchr_decode(int fd, int supported, u32 value, int width)
{
	const gsc_reg_def_t*	bcfgr;
	int						ret;

	if (supported)
	{
		bcfgr	= _find_reg(AI32SSC1M_GSC_BCFGR, _gsc);
		ret		= bcfgr
				? (((bcfgr->value & 0xFFF) > 4) ? 1 : 0)
				: 0;
	}
	else
	{
		ret	= 0;
		gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 31, 16, 1, NULL,	"Reserved"		);
		gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 15,  0, 1, NULL,	"High Count"	);
	}

	return(ret);
}



//*****************************************************************************
static int _ecdr_decode(int fd, int supported, u32 value, int width)
{
	const gsc_reg_def_t*	bcfgr;
	int						ret;

	if (supported)
	{
		bcfgr	= _find_reg(AI32SSC1M_GSC_BCFGR, _gsc);
		ret		= bcfgr
				? (((bcfgr->value & 0xFFF) >= 0x010) ? 1 : 0)
				: 0;
	}
	else
	{
		ret	= 0;
		gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 19,  0, 1, NULL,	"Divisor"	);
		gsc_reg_field_show(NAME_WIDTH, FIELD_WIDTH, value, 31, 20, 1, NULL,	"Reserved"	);
	}

	return(ret);
}



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	ai32ssc1m_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* ai32ssc1m_reg_get_def_id(u32 reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	ai32ssc1m_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* ai32ssc1m_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	if (index < 0)
		def	= NULL;
	else if (index >= (SIZEOF_ARRAY(_gsc) - 1))
		def	= NULL;
	else
		def	= &_gsc[index];

	return(def);
}



/******************************************************************************
*
*	Function:	ai32ssc1m_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* ai32ssc1m_reg_get_desc(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	ai32ssc1m_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* ai32ssc1m_reg_get_name(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}



/******************************************************************************
*
*	Function:	ai32ssc1m_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int ai32ssc1m_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, ai32ssc1m_reg_read);
	return(errs);
}



