// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/utils/util_vpd_read.c $
// $Rev: 44165 $
// $Date: 2018-11-20 14:44:06 -0600 (Tue, 20 Nov 2018) $

// 18AI32SSC1M: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ai32ssc1m_vpd_read
*
*	Purpose:
*
*		Provide a visual wrapper for the AI32SSC1M_IOCTL_VPD_READ service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		type	The VPN type data to read. Use -1 for all.
*
*		get		The data is returned here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai32ssc1m_vpd_read(int fd, int index, int verbose, gsc_vpd_type_t type, gsc_vpd_t* get)
{
	char		buf[64];
	int			errs	= 0;
	const char*	psz		= "";
	int			ret;
	gsc_vpd_t	set;

	if (verbose)
	{
		switch (type)
		{
			default:

				errs++;
				sprintf(buf, "UNKNOWN TYPE (%d)", type);
				psz	= buf;
				break;

			case GSC_VPD_TYPE_MODEL_NUMBER:		psz	= "Model Number";	break;
			case GSC_VPD_TYPE_NAME:				psz	= "Product Name";	break;
			case GSC_VPD_TYPE_SERIAL_NUMBER:	psz	= "Serial Number";	break;
		}

		gsc_label_index(psz, index);
	}

	memset(&set, 0, sizeof(set));
	set.type	= type;
	ret			= ai32ssc1m_ioctl(fd, AI32SSC1M_IOCTL_VPD_READ, &set);
	errs		+= ret ? 1 : 0;

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", set.data);

	if (get)
		get[0]	= set;

	return(errs);
}


